#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#define EPS 1.0e-10
#define FPMIN 1.0e-30
#define MAXIT 10000
#define PI 3.141592653589793

#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))

static void nrerror(char error_text[]); 

void bessel(float *rj, float *ry, int nl, float x, float limit)
{
	int i,isign,l;
	double a,b,br,bi,c,cr,ci,d,del,del1,den,di,dlr,dli,dr,e,f,fact,fact2,
		fact3,ff,gam,gam1,gam2,gammi,gampl,h,p,pimu,pimu2,q,r,rjl,
		rjl1,rjmu,rjp1,rjpl,rjtemp,ry1,rymu,rymup,rytemp,sum,sum1,
		temp,w,x2,xi,xi2,xmu,xmu2,xnu;

	if (x <= 0.0 || nl < 0) nrerror("bad arguments in bessel");
/* modified BESSJY for particular case where xnu = int+0.5, see numrec  */ 
xmu = 0.50; 
xnu = xmu+nl; 
	xmu2=xmu*xmu;
	xi=1.0/x;
	xi2=2.0*xi;
	w=xi2/PI;
	isign=1;
	h=xnu*xi;
	if (h < FPMIN) h=FPMIN;
	b=xi2*xnu;
	d=0.0;
	c=h;
	for (i=1;i<=MAXIT;i++) {
		b += xi2;
		d=b-d;
		if (fabs(d) < FPMIN) d=FPMIN;
		c=b-1.0/c;
		if (fabs(c) < FPMIN) c=FPMIN;
		d=1.0/d;
		del=c*d;
		h=del*h;
		if (d < 0.0) isign = -isign;
		if (fabs(del-1.0) < EPS) break;
	}
	if (i > MAXIT) nrerror("x too large in bessjy; try asymptotic expansion");
	rjl=isign*FPMIN;
	rjpl=h*rjl;
	rj[nl]=rjl1=rjl;
	rjp1=rjpl;
	fact=xnu*xi;
        if (x > limit) {
	    for (l=nl;l>=1;l--) {
		rj[l-1]=rjtemp=fact*rjl+rjpl;
		fact -= xi;
		rjpl=fact*rjtemp-rjl;
		rjl=rjtemp;
	    }
        } else { 
	    for (l=nl;l>=1;l--) {
		rj[l-1]=rjtemp=0.5*(fact*rjl+rjpl)*x/l;
		fact -= xi;
		rjpl=fact*rjtemp-0.5*rjl*x/l;
		rjl=rjtemp;
	    }
        } 
	if (rjl == 0.0) rjl=EPS;
	f=rjpl/rjl;
	a=0.25-xmu2;
	p = -0.5*xi;
	q=1.0;
/* no need to iterate for CF2, according to numrec, when nu = int+0.5  */ 
	gam=(p-f)/q;
	rjmu=sqrt(w/((p-f)*gam+q));
	rjmu=SIGN(rjmu,rjl);
	ry[0] = rymu=rjmu*gam;
	rymup=rymu*(p+q/gam);
	ry1=xmu*xi*rymu-rymup;
        fact=rjmu/rjl; 
/* rescale J_n  */ 
	for (i=0;i<=nl;i++) rj[i] *= fact; 
for (i=0;i<=nl;i++) printf("%d %e\n", i, rj[i]);
        if (x > limit) {
/* compute Y_n  */ 
	    ry[1] = ry1; 
printf("%d %e\n", 0, rymu); 
printf("%d %e\n", 1, ry1); 
	    for (i=1;i<nl;i++) {
		ry[i+1]=rytemp=(xmu+i)*xi2*ry1-rymu;
		rymu=ry1;
		ry1=rytemp;
printf("%d %e\n", i+1, rytemp); 
            } 
        } else { 
/* compute Y_n * x^n / n!  to avoid overfloats for small x  */ 
            ry1 *= 0.5*x; ry[1] = ry1;  
printf("%d %e\n", 0, rymu); 
printf("%d %e\n", 1, ry1); 
	    for (i=1;i<nl;i++) {
		ry[i+1]=rytemp=(xmu+i)*ry1/(i+1)-0.25*x*x*rymu/(i+1)/i;
		rymu=ry1;
		ry1=rytemp;
printf("%d %e\n", i+1, rytemp); 
            } 
        } 
}
#undef EPS
#undef FPMIN
#undef MAXIT
#undef PI


static void nrerror(char error_text[])
{
        printf("Numerical Recipes run-time error...\n");
        printf("%s\n",error_text);
        printf("...now exiting to system...\n");
        exit(1);
}

