
#include <stdio.h> 
#include <string.h>
#include "structures.h" 
#include "mem_alloc.h"
#include "graf.h" 
#include "file.h" 

void get_spec_dims(spec_info_t spec_info[], int nobs, int *npns, int datsw[]); 
void get_data_to_plot(spec_info_t **spec_info, int nfile, int *nobs, int *photo, 
                      char name[]); 


int main(void) 
{
    int    j, k, p, q, n, m, nfile, getname, open_ok, datsw[4], *nobs, photo, dyn; 
    float  **I, **Q, **U, **V; 
    char   name[80], *str, com[120], com1[120], title[80]; 
    spec_info_t **info; 

    getname = 1; 

    /* Reads in the spectra */ 
    printf("How many data file to read : "); 
    scanf("%d", &nfile);  
    I     = (float **) svector(nfile, (int) sizeof(float *)); 
    Q     = (float **) svector(nfile, (int) sizeof(float *)); 
    U     = (float **) svector(nfile, (int) sizeof(float *)); 
    V     = (float **) svector(nfile, (int) sizeof(float *)); 
    nobs  = ivector(0, nfile-1); 
    info  = (spec_info_t **) svector(nfile, (int) sizeof(spec_info_t *)); 

    for (j = 0; j < nfile; j++) 
        do { 
            if (j == 0) str = com; 
            else        str = com1; 
            open_ok = read_spec(I+j, Q+j, U+j, V+j, nobs+j, info+j, name, getname, str); 
            if (open_ok == 1) printf("File not found. "); 
        } while (open_ok == 1); 

//    gfield(I, Q, U, V, info, nfile, nobs, title); 

    /* Plots the spectra */ 
    get_data_to_plot(info, nfile, nobs, &photo, name); 
    if (!photo) { 
        sprintf(title, "%s (%s profiles)", com, name); 
sprintf(title, "%s", com); 
title[0] = '\0'; 
//sprintf(title, "Rotational modulation of Stokes V signature"); 
        printf("Dynamic spectrum of profile plot (1/0) : "); 
        scanf("%d", &dyn); 
        if (dyn) 
            gdyn(I, Q, U, V, info, nfile, nobs, title); 
        else 
            gspec(I, Q, U, V, info, nfile, nobs, title); 
    } else { 
        sprintf(title, "%s (photometry)", com); 
        gphot(I, Q, U, V, info, nfile, nobs, title); 
    }

    /* Release memory */ 
    for (j = 0; j < nfile; j++) { 
        get_spec_dims(info[j], nobs[j], &n, datsw); 
        if (datsw[0] == 1) free_vector(I[j], 0, n-1); 
        if (datsw[1] == 1) free_vector(Q[j], 0, n-1); 
        if (datsw[2] == 1) free_vector(U[j], 0, n-1); 
        if (datsw[3] == 1) free_vector(V[j], 0, n-1); 
        free_svector((void *) info[j]); 
    }
    free_svector((void *) I,    0, nfile-1); 
    free_svector((void *) Q,    0, nfile-1); 
    free_svector((void *) U,    0, nfile-1); 
    free_svector((void *) V,    0, nfile-1); 
    free_ivector(nobs,          0, nfile-1); 
    free_svector((void *) info, 0, nfile-1); 
}


void get_data_to_plot(spec_info_t **spec_info, int nfile, int *nobs, int *photo, 
                      char name[]) 
{
    int    n,m,j,k,**flag,nlines,*type,*used; 
    char   **line_names, answer[2];

    flag = (int **) svector(nfile, (int) sizeof(int *)); 
    for (n = 0; n < nfile; n++) { 
        flag[n] = ivector(0, nobs[n]-1); 
        for (j = 0; j < nobs[n]; j++) flag[n][j] = 0; 
    }

    /* Count the number of different type of observation in all data files */ 
    nlines = 0; 
    for (n = 0; n < nfile; n++) 
        for (j = 0; j < nobs[n]; j++) 
            if (!flag[n][j]) {
                for (m = n; m < nfile; m++) 
                    for (k = j*(m==n); k < nobs[m]; k++) 
                        if (!strcmp(spec_info[m][k].line_name,spec_info[n][j].line_name)) 
                            flag[m][k] = 1;
                nlines++; 
            }

    type = ivector(0, nlines-1); 
    line_names = (char **) svector(nlines, (int) sizeof(char *)); 
    for (n = 0; n < nlines; n++) 
        line_names[n] = (char *) svector(80, (int) sizeof(char)); 

    /* Record the name and type of each type of observation */ 
    for (n = 0; n < nfile; n++) for (j = 0; j < nobs[n]; j++) flag[n][j] = 0; 
    nlines = 0; 
    for (n = 0; n < nfile; n++) 
        for (j = 0; j < nobs[n]; j++) 
            if (!flag[n][j]) {
                sprintf(line_names[nlines], "%s", spec_info[n][j].line_name); 
                type[nlines] = spec_info[n][j].photometric; 
                for (m = n; m < nfile; m++) 
                    for (k = j*(m==n); k < nobs[m]; k++) 
                        if (!strcmp(spec_info[m][k].line_name,spec_info[n][j].line_name))
                            flag[m][k] = 1;
                nlines++; 
            }

    /* Ask user for observation to plot */ 
    *photo = - 1; 
    used = ivector(0, nlines-1); 
    if (nlines == 1) {
        used[0] = 1; 
        *photo = type[0]; 
        sprintf(name, "%s", line_names[0]); 
    } else { 
        for (k = 0; k < nlines; k++) used[k] = 0; 
        do {
            for (k = 0; k < nlines; k++) {
                if ((*photo == 1) && (type[k] == 0)) continue; 
                printf("Do you want to plot the %s observations (y/n) : ",line_names[k]);
                scanf("%1s",answer);
                if ((answer[0] == 'Y') || (answer[0] == 'y')) 
                    if (*photo < 0) {
                        used[k] = 1; 
                        *photo = type[k]; 
                        sprintf(name, "%s", line_names[k]); 
                        if (*photo == 0) break; 
                    } else {
                        if (type[k] == 0) 
                            printf("You can't mix spectroscopy and photometry\n"); 
                        else 
                            used[k] = 1; 
                    }
            }
        } while (*photo < 0); 
    }

    /* Adjust spec_info */ 
    for (n = 0; n < nfile; n++) 
        for (j = 0; j < nobs[n]; j++) 
            for (k = spec_info[n][j].selected = 0; k < nlines; k++) 
                if (used[k] && !strcmp(spec_info[n][j].line_name,line_names[k])) 
                    spec_info[n][j].selected = 1; 

    for (n = 0; n < nfile; n++)  free_ivector(flag[n], 0, nobs[n]-1); 
    free_svector((void *) flag); 
    for (n = 0; n < nlines; n++) free_svector((void *) line_names[n]); 
    free_svector((void *) line_names); 
    free_ivector(type, 0, nlines-1); 
    free_ivector(used, 0, nlines-1); 
}


void get_spec_dims(spec_info_t spec_info[], int nobs, int *npns, int datsw[])
{
    int j,k;

    for (k = 0; k < 4; k++)
        datsw[k] = 0;

    *npns = 0;

    for (j = 0; j < nobs; j++) {
        *npns += spec_info[j].nspec;
        for (k = 0; k < 4; k++)
            if (spec_info[j].datsw[k]) datsw[k] = 1;
    }
}
