#include <stdio.h> 
#include <stdlib.h> 
#include <math.h>
#include "structures.h" 
#include "mem_alloc.h"
#include "plot.h" 

static void fit(float *lam, float *spec, int *good, int nspec, int sub);
static void linear(float *x, float *y, int npt, float *lam, float *data, int nspec); 
static int locate(float x, float array[], int n); 
static void get_datsw(int datsw[], int *n_data, int batch); 
static void indexx(int n, float *arrin, int *indx); 
static void cp_phot(float *I, spec_info_t *info, int nobs, float **phases, 
                    float ***color, int *nphot, int photsw[]); 


void gspec(float **I, float **Q, float **U, float **V, spec_info_t **info, int nfile, 
           int *nobs, char *title)
{
    int   j, k, l, q, p, m, n, maxspec, datsw_new[4], ntot, nsppg, dev, *nphase, nsp, 
          ptot, *nn, *kk, nwin; 
int nnn, kkk, ppp, tt; 
    float xmin, ymin, xmax, ymax, **lam, fac, **pt, shift, step, tmp, **zero, xbar[2], ybar[2]; 
float *xxx, *yyy, *mmm,*jl, *ju; 
    char  labx[80] = {"Wavelength (nm)"}, lab2x[80] = {"Velocity (km/s)"}, txt[20], 
          laby[80], stokes[10] = {"IQUV"}, com[120]; 

    lam = (float **) svector(nfile, (int) sizeof(float *)); 
    zero = (float **) svector(nfile, (int) sizeof(float *)); 
    nphase = ivector(0, nfile-1); 
    xmin = 1.0e36; xmax = -1.0e36; maxspec = 0; 
    datsw_new[0] = datsw_new[1] = datsw_new[2] = datsw_new[3] = 1; 

    for (j = 0; j < nfile; j++) { 
        nphase[j] = ntot = 0; 
        for (k = 0; k < nobs[j];  k++) {
            ntot += info[j][k].nspec; 
            if (!info[j][k].photometric && info[j][k].selected) {
                (nphase[j])++; 
//                for (p = 0; p < 4; p++) datsw_new[p] *= info[j][k].datsw[p]; 
                for (p = 0; p < 4; p++)
                    if (datsw_new[p] == 0) datsw_new[p] += info[j][k].datsw[p];
            }
        }
        if (nphase[j] > maxspec) maxspec = nphase[j]; 

        lam[j] = vector(0, ntot-1); 
        zero[j] = vector(0, ntot-1); 
        for (k = n = 0; k < nobs[j]; n += info[j][k].nspec, k++) 
            if (!info[j][k].photometric && info[j][k].selected) {
                step = (info[j][k].lamu-info[j][k].laml) / (info[j][k].nspec-1); 
/*
                for (p = 0; p < info[j][k].nspec; p++) 
                    lam[j][n+p] = (info[j][k].laml + step*p); 
*/
                for (p = 0; p < info[j][k].nspec; p++) 
                    lam[j][n+p] = 299792.5*((info[j][k].laml + step*p)/info[j][k].line_centre - 1.0); 
                if (xmin > lam[j][n]) xmin = lam[j][n]; 
                if (xmax < lam[j][n+p-1]) xmax = lam[j][n+p-1]; 
            } 
    }
    tmp = xmax-xmin; xmin -= 0.25*tmp; xmax += 0.65*tmp; 
    nsppg = (float) maxspec/20.0+0.4999;  if (nsppg < 1) nsppg = 1; 
    nsppg = (float) maxspec/nsppg+0.999; 
/*
nsppg = 98; 
nsppg = 17; 
nsppg = 129; 
nsppg = 32; 
nsppg = 23; 
nsppg = 14; 
nsppg = 16; 
nsppg = 43; 
nsppg = 16; 
*/
/*
nsppg = 15; 
nsppg = 16; 
nsppg = 10; 
nsppg = 21; 
nsppg = 11; 
nsppg = 41; 
nsppg = 30; 
nsppg = 6; 
nsppg = 22; 
nsppg = 12; 
nsppg = 11; 
*/
nsppg = 14; 
nsppg = 9; 
nsppg = 10; 
nsppg = 11; 
nsppg = 28; 
nsppg = 6; 

    get_datsw(datsw_new, &n, 0); 
    for (k = m = 0; k < 4; k++) m += k * (datsw_new[k] == 1); 
    if      (m == 0) pt = I; 
    else if (m == 1) pt = Q; 
    else if (m == 2) pt = U; 
    else if (m == 3) pt = V; 
    if (m > 0) fac = 100.0; 
    else       fac = 1.0; 

maxspec = 0; 
    for (j = 0; j < nfile; j++) {
        tt = 0;
        for (k = 0; k < nobs[j];  k++) {
            if (!info[j][k].photometric && info[j][k].selected) {
                for (p = 0; p < 4; p++)
                    if ((datsw_new[p] == 1) && (info[j][k].datsw[p] == 1)) tt++;
            }
        }
        if (tt > maxspec) maxspec = tt; 
printf("%d %d %d\n", j, tt, maxspec);   
    } 


    if (maxspec > 1) { 
        printf("Shift between spectra : "); 
        scanf("%f", &shift); 
    } else 
        shift = 0.0; 
/*
tmp=0.0; xmax=-1.0e36;
for (j = 0; j < nfile; j++)
    for (k = n = l = 0; k < nobs[j]; n += info[j][k].nspec, k++)
        if (!info[j][k].photometric && info[j][k].selected) {
ymin = 1.0e10; ymax = -1.0e10;
            for (p = 0; p < info[j][k].nspec; p++) { 
                    tmp = pt[j][n+p]*fac/info[j][k].continuum; 
                    if (ymin > tmp) ymin = tmp; 
                    if (ymax < tmp) ymax = tmp; 
            } 
if (ymax-ymin > xmax) xmax = ymax-ymin; 
        } 
printf(">>> %f\n", xmax); 
*/

    ymin = 1.0e36; ymax = -1.0e36; 
    for (j = 0; j < nfile; j++) 
        for (k = n = l = 0; k < nobs[j]; n += info[j][k].nspec, k++) 
            if (!info[j][k].photometric && info[j][k].selected && info[j][k].datsw[m]) {
                q = l%nsppg; 
/*
if (l < 129)      q = l; 
else if (l < 225) q = l-129; 
else              q = l-225; 
if (l < 32)      q = l; 
else if (l < 56) q = l-32; 
else             q = l-56; 

if (l < 11)      q = l; 
else if (l < 21) q = l-11; 
else             q = l-21; 

if (l < 40)      q = l; 
else if (l < 79) q = l-40; 
else             q = l-79; 

if (l < 44)      q = l; 
else if (l < 83) q = l-44; 
else             q = l-83; 

if (l < 13)       q = l; 
else if (l < 27)  q = l-13; 
*/
                for (p = 0; p < info[j][k].nspec; p++) {
                    pt[j][n+p] = tmp = pt[j][n+p]*fac/info[j][k].continuum - shift*q; 
                    zero[j][n+p] = -shift*q; 
if (m == 0) zero[j][n+p] += 1; 
                    if (ymin > tmp) ymin = tmp; 
                    if (ymax < tmp) ymax = tmp; 
                }
                l++; 
            } 
    tmp = ymax-ymin; 
    ymin -= 0.05*tmp; ymax += 0.05*tmp; 

    printf("Screen or laser print (0/1) : "); 
    scanf("%d", &dev); 
    set_device(dev); 
    sprintf(laby, "%c/I\\Dc\\U",stokes[m]); 
    if (m > 0) sprintf(laby, "%s (%%)",laby); 

/*
ymin = 0.44;   ymax = 1.01; 
ymin = -2.7; ymax = 0.2;
ymin = -3.4; ymax = 0.2; 
ymin = 0.57;   ymax = 1.01; 
ymin = -3.3; ymax = 0.3;

ymin = 0.35;   ymax = 1.02; 
ymin = -3.4; ymax = 0.2; 
ymin = 0.53;   ymax = 1.01; 
ymin = -3.3; ymax = 0.3;
ymin = 0.55;   ymax = 1.02; 
ymin = 0.46;   ymax = 1.01; 
*/
/*
ymin = -3.0; ymax = 0.2;
ymin = 0.65; ymax = 1.01; 
ymin = 0.52;   ymax = 1.01; 
ymin = -3.9; ymax = 0.3;
ymin = -3.3; ymax = 0.3;
ymin = -0.99999; ymax = 0.1999;
ymin = 0.75001;   ymax = 1.02; 
ymin = 0.2;   ymax = 1.02; 
ymin = -2.45; ymax = 0.2;
ymin = -5.5; ymax = 0.5;
*/
//ymin = -0.7999; ymax = 0.0999;
//ymin = -0.57; ymax = 0.02;
//ymin = 0.910001; ymax = 1.009999;
//ymin = 0.05; ymax = 1.05;
//ymin = -4.4; ymax = 0.4;
//ymin = -7.5; ymax = 2.5;
//ymin = -130.0; ymax = 10.0;
//ymin = -7.4; ymax = 2.4;
//ymin = -260.0; ymax = 20.0;
//ymin = -125.0; ymax = 25.0;
//ymin = -41.0; ymax = 9.0;
//ymin = 0.05; ymax = 1.05;
//ymin = -3.99; ymax = 2.2;
//ymin = -130.0; ymax = 10.0;
//ymin = -7.5; ymax = 2.5;
//ymin = -4.4; ymax = 0.6;
//ymin = -6.5; ymax = 0.5;
//ymin = -2.6; ymax = 0.3;
//ymin = -5.4; ymax = 0.5;
/*
nnn=27; 
xxx=vector(0, nnn-1); 
yyy=vector(0, nnn-1); 
mmm=vector(0, nnn-1); 
for (kkk = 0; kkk < nnn; kkk++) { xxx[kkk] = (kkk-nnn/2)*3.0; mmm[kkk] = 0.0; }
ppp = 0; 
*/
/*
if      (m == 0) { ymin =  0.12; ymax = 1.0999; } 
else if (m == 1) { ymin = -4.4; ymax = 0.4999; }
else if (m == 2) { ymin = -4.4; ymax = 0.4999; }
else if (m == 3) { ymin = -44.0; ymax = 4.999; }
*/

    gets(com);  
    kk = ivector(0, nfile-1); 
    nn = ivector(0, nfile-1); 
    ptot = 0; 
    for (j = 0; j < nfile; j++) kk[j] = nn[j] = 0; 

nwin = (float) maxspec/nsppg+0.99999; 
if (nwin == 1) tmp = (float) maxspec/nsppg; 
else           tmp = 1.0; 
tmp=1.0; 
printf("%d %d %d\n", maxspec, nsppg, nwin); 
nwin=4; 
init_multiplot(nwin,1); 
cpgpap(0.0,4.0*tmp/nwin); 
cpgpap(0.0,3.0*tmp/nwin); 


    for (ptot = 0; ptot < maxspec; ptot += nsppg) {  
/*
if (ptot == 0)        nsppg = 129; 
else if (ptot == 129) nsppg = 96; 
else                  nsppg = 98; 
if (ptot == 0)        nsppg = 32; 
else                  nsppg = 24; 

if (ptot == 0)       nsppg = 11; 
else if (ptot == 11) nsppg = 10; 
else                 nsppg = 11; 

if (ptot == 0)       nsppg = 40; 
else if (ptot == 40) nsppg = 39; 
else                 nsppg = 43; 

if (ptot == 0)       nsppg = 44; 
else if (ptot == 44) nsppg = 39; 
else                 nsppg = 40; 

if (ptot == 0)       nsppg = 13; 
else if (ptot == 13) nsppg = 14; 
*/
/*
        init_plot(xmin, xmax, ymin, ymax, lab2x, laby, title, 0, 0); 
*/
        start_plot(xmin, xmax, ymin, ymax, lab2x, laby, " ", 0, -2); 

        for (j = 0; j < nfile; j++) { 
            for (p = 0; kk[j] < nobs[j] && p < nsppg; nn[j] += info[j][kk[j]].nspec, kk[j]++) 
                if (!info[j][kk[j]].photometric && info[j][kk[j]].selected && info[j][kk[j]].datsw[m]) {
/*
linear(xxx,yyy,nnn,lam[j]+nn[j],pt[j]+nn[j],info[j][kk[j]].nspec); 
for (kkk = 0; kkk < nnn; kkk++) mmm[kkk] += yyy[kkk]; 
ppp++; 
*/
/*
                    if (j == 0) plot_sym(lam[j]+nn[j], pt[j]+nn[j], info[j][kk[j]].nspec, 20); 
*/
                    if (j == 0) {
//                    if (j < 2) {
cpgsci(2);
//cpgsci(3-j);
                        plot(lam[j]+nn[j], pt[j]+nn[j], info[j][kk[j]].nspec, 51); 
cpgsci(1);
/*
                        plot_sym(lam[j]+nn[j], pt[j]+nn[j], info[j][kk[j]].nspec, 20); 
*/
                    } else        {

                        plot(lam[j]+nn[j], pt[j]+nn[j], info[j][kk[j]].nspec, (j-1)%4); 
//                        plot(lam[j]+nn[j], pt[j]+nn[j], info[j][kk[j]].nspec, j%4); 
/*
                        plot(lam[j]+nn[j], pt[j]+nn[j], info[j][kk[j]].nspec, j%4+1); 
*/
                    } 
/*
if (j == 0) {
jl=vector(0, info[j][kk[j]].nspec-1); 
ju=vector(0, info[j][kk[j]].nspec-1); 
for (kkk=0; kkk<info[j][kk[j]].nspec; kkk++) { 
jl[kkk] = pt[j][nn[j]+kkk]-fac/info[j][kk[j]].SN; 
ju[kkk] = pt[j][nn[j]+kkk]+fac/info[j][kk[j]].SN; 
}
plot_erry(lam[j]+nn[j], jl, ju, info[j][kk[j]].nspec); 
free_vector(jl,0,info[j][kk[j]].nspec-1); 
free_vector(ju,0,info[j][kk[j]].nspec-1); 
} 
*/
/*
printf("\n***\n%d 2\n", info[j][kk[j]].nspec); 
for (k = 0; k < info[j][kk[j]].nspec; k++) 
printf("%.4f %.4e %.4e\n", lam[j][nn[j]+k], I[j][nn[j]+k], 1.0/info[j][kk[j]].SN); 
*/
/*
printf("\n***\n%d 6\n", info[j][kk[j]].nspec); 
for (k = 0; k < info[j][kk[j]].nspec; k++) 
printf("%f %e %e %e %e 0.0 %e\n", lam[j][nn[j]+k], I[j][nn[j]+k], 1.0/info[j][kk[j]].SN,0.01*V[j][nn[j]+k],1.0/info[j][kk[j]].SN,1.0/info[j][kk[j]].SN); 
*/
                    if (m) plot(lam[j]+nn[j], zero[j]+nn[j], info[j][kk[j]].nspec, 4); 
//plot(lam[j]+nn[j], zero[j]+nn[j], info[j][kk[j]].nspec, 4); 
                    if (j == nfile-1) {
/*
                        sprintf(txt, "phase %.3f", 0.5*(info[j][kk[j]].phasec+info[j][kk[j]].phasef));
*/
kkk=0.5*(info[j][kk[j]].phasec+info[j][kk[j]].phasef);
//                        sprintf(txt, "%.3f", 0.5*(info[j][kk[j]].phasec+info[j][kk[j]].phasef)-kkk+(info[j][kk[j]].phasec+info[j][kk[j]].phasef<0.0));
                        sprintf(txt, "%.3f", 0.5*(info[j][kk[j]].phasec+info[j][kk[j]].phasef)+1.0);
xbar[0] = xbar[1] = xmin+0.1*(xmax-xmin); 
ybar[0] = zero[j][nn[j]]-150.0/info[j][kk[j]].SN;  
ybar[1] = zero[j][nn[j]]+150.0/info[j][kk[j]].SN; 
if (m) plot(xbar,ybar,2,51); 
ybar[0] = ybar[1] = zero[j][nn[j]]-150.0/info[j][kk[j]].SN; 
xbar[0] = xmin+0.095*(xmax-xmin); 
xbar[1] = xmin+0.105*(xmax-xmin); 
if (m) plot(xbar,ybar,2,51); 
ybar[0] = ybar[1] = zero[j][nn[j]]+150.0/info[j][kk[j]].SN; 
xbar[0] = xmin+0.095*(xmax-xmin); 
xbar[1] = xmin+0.105*(xmax-xmin); 
if (m) plot(xbar,ybar,2,51); 
                        if (m) write_text(txt, xmax-0.17*(xmax-xmin), zero[j][nn[j]]); 
                        else   write_text(txt, xmax-0.17*(xmax-xmin), pt[j][nn[j]+info[j][kk[j]].nspec-1]); 
//else write_text(txt, xmax-0.17*(xmax-xmin), zero[j][nn[j]]); 

/*

                        sprintf(txt, "%.3f", 0.5*(info[j][kk[j]].phasec+info[j][kk[j]].phasef));
                        if (p%2 == 0) write_text(txt, xmax-0.17*(xmax-xmin), pt[j][nn[j]+info[j][kk[j]].nspec-1]); 
*/

                    }
                    p++; 
                }
        } 
/*
        end_plot(); 
*/
        if (ptot+nsppg < maxspec) { 
            printf("Type return to get the remaining profiles \n"); 
            gets(com);  
        } 
    } 
    end_plot(); 
/*
for (kkk = 0; kkk < nnn; kkk++) mmm[kkk] /= (float) ppp; 
printf("\n***\n%d 1\n", nnn); 
for (k = 0; k < nnn; k++) printf("%f %e\n", xxx[k], mmm[k]); 
free_vector(xxx, 0, nnn-1); 
free_vector(yyy, 0, nnn-1); 
free_vector(mmm, 0, nnn-1); 
*/
    free_ivector(kk, 0, nfile-1); 
    free_ivector(nn, 0, nfile-1); 

    for (j = 0; j < nfile; j++) { 
        for (k = n = 0; k < nobs[j]; k++) n += info[j][k].nspec; 
        free_vector(lam[j], 0, n-1); 
        free_vector(zero[j], 0, n-1); 
    }
    free_ivector(nphase,          0, nfile-1); 
    free_svector((void *) lam); 
    free_svector((void *) zero); 
}


void gphot(float **I, float **Q, float **U, float **V, spec_info_t **info, int nfile, 
           int *nobs, char *title)
{
    int    j, k, k1, k2, p, device, n, sign, **photsw, phot[5], *nphot; 
    float  xmin, xmax, ymin, ymax, max, min, tmp, shift, ***color, **phases; 
    char   labx[80] = {"Rotational phase"}, laby[80] = {""}, johnson[6] = {"UBVRI"}, 
           string[80]; 

    color  = (float ***) svector(nfile, (int) sizeof(float **)); 
    phases = (float **)  svector(nfile, (int) sizeof(float *)); 
    photsw = imatrix(0, nfile-1, 0, 4); 
    nphot  = ivector(0, nfile-1); 

/* Put color measurements in array color */ 
    for (k = 0; k < 5; k++) phot[k] = 1;  
    for (j = 0; j < nfile; j++) {
        cp_phot(I[j], info[j], nobs[j], phases+j, color+j, nphot+j, photsw[j]); 
        for (k = 0; k < 5; k++) phot[k] *= photsw[j][k]; 
    }

/* Choose the main color index */ 
    for (k = 0, sign = 1, n = 2; k < 5; k++, sign *= -1) 
        if (phot[(n += sign*k)] == 1) break; 

    ymax = -1.0e30; ymin = 1.0e30; 
    for (j = 0; j < nfile; j++) 
        for (p = 0; p < nphot[j]; p++) {
            tmp = color[j][n][p]; 
            if (tmp > ymax) ymax = tmp; 
            if (tmp < ymin) ymin = tmp; 
        }
    tmp = ymax-ymin; ymax += 0.25*tmp; 
    sprintf(laby, "%c        ", johnson[n]); 

/* Rearrange all other color indexes */ 
    for (k = 0; k < 5; k++) 
        if (phot[k] && (k != n)) {
            if (k < n) {
                k1 = k; 
                k2 = n; 
            } else { 
                k1 = n; 
                k2 = k; 
            } 
            max = -1.0e30; min = 1.0e30; 
            for (j = 0; j < nfile; j++) 
                for (p = 0; p < nphot[j]; p++) {
                    tmp = color[j][k][p] = color[j][k1][p] - color[j][k2][p]; 
                    if (tmp > max) max = tmp; 
                    if (tmp < min) min = tmp; 
                }
            for (j = 0; j < nfile; j++) 
                for (p = 0; p < nphot[j]; p++) color[j][k][p] += ymax-min; 
            ymax += max-min; 
            tmp = ymax-ymin; ymax += 0.10*tmp; ymin -= 0.10*tmp;  
            sprintf(string, "%s", laby); 
            sprintf(laby, "%c-%c            %s", johnson[k1], johnson[k2], string); 
        }

/* Plot on screen of paper */ 
    printf("Screen display or paper copy (0/1) : "); 
    scanf("%d",&device); 
    set_device(device); 

    init_plot(0.0,1.0,ymax,ymin,labx,laby,title,0,0); 
    for (k = 0; k < 5; k++) 
        if (phot[k]) 
            for (j = 0; j < nfile; j++) { 
                if (j == 0) plot_sym(phases[j],color[j][k],nphot[j],3); 
                else        plot(phases[j],color[j][k],nphot[j],j); 
            }
    end_plot();

    for (j = 0; j < nfile; j++) { 
        free_matrix(color[j], 0, 4, 0, nphot[j]-1); 
        free_vector(phases[j], 0, nphot[j]-1); 
    }
    free_svector((void *) color); 
    free_svector((void *) phases); 
    free_ivector(nphot, 0, nfile-1); 
    free_imatrix(photsw, 0, nfile-1, 0, 4); 
}


static void get_datsw(int datsw[], int *n_data, int batch)
{
    char answer[20];
    int c,i,j,new_datsw[4];

    *n_data = 0;
    for (i = 0; i < 4; i++) {
        new_datsw[i] = 0;
        if (datsw[i] == 1)
            ++(*n_data);
    }
    if (*n_data == 0) {
        fprintf(stderr,"Error: No Stokes parameters available\n");
        exit(1);
    }
    if (*n_data == 1)
        return;

    *n_data = 0;
    printf("Select Stokes parameter to plot (one of ");
    if (datsw[0] == 1)  printf("I");
    if (datsw[1] == 1)  printf("Q");
    if (datsw[2] == 1)  printf("U");
    if (datsw[3] == 1)  printf("V");
    printf(") : ");
    scanf("%20s", answer);
    j = 0;
    while (answer[j] != '\0') {
        c = answer[j];
        if ((datsw[0] == 1) && ((c == 'I') || (c == 'i'))) {
            if (new_datsw[0] == 0) {
                ++(*n_data);
                if (batch == 1) printf("I");
            }
            new_datsw[0] = 1;
        } else if ((datsw[1] == 1) && ((c == 'Q') || (c == 'q'))) {
            if (new_datsw[1] == 0) {
                ++(*n_data);
                if (batch == 1) printf("Q");
            }
            new_datsw[1] = 1;
        } else if ((datsw[2] == 1) && ((c == 'U') || (c == 'u'))) {
            if (new_datsw[2] == 0) {
                ++(*n_data);
                if (batch == 1) printf("U");
            }
            new_datsw[2] = 1;
        } else if ((datsw[3] == 1) && ((c == 'V') || (c == 'v'))) {
            if (new_datsw[3] == 0) {
                ++(*n_data);
                if (batch == 1) printf("V");
            }
            new_datsw[3] = 1;
        } else if ((c != ' ') && (c != ','))
            fprintf(stderr,"Unknown parameter <%c> ignored\n", c);

        j++;
    }
    if (batch == 1) printf("\n");
    if (*n_data == 0)
        if (batch == 0) {
            fprintf(stderr,"You must choose one parameter.  Try again\n");
            get_datsw(datsw,n_data,batch);
        } else {
            fprintf(stderr,"You must choose one parameter.\n");
            exit(1);
        }
    else if (*n_data > 1)
        if (batch == 0) {
            fprintf(stderr,"You must choose only one parameter.  Try again\n");
            get_datsw(datsw,n_data,batch);
        } else {
            fprintf(stderr,"You must choose only one parameter.\n");
            exit(1);
        }
    for (j = 0; j < 4; j++)
        datsw[j] = new_datsw[j];
}


static void cp_phot(float *I, spec_info_t *info, int nobs, float **phases, 
                    float ***color, int *nphot, int photsw[]) 
{
    int   j, k, p, n[5], m[5], nn, npt, *indx, jjj; 
    float **phot, **phi, *tmp1, *tmp2, max; 
    char  johnson[6] = {"UBVRI"}; 

    /* Allocate temporary memory */
    for (k = 0; k < 5; k++) n[k] = photsw[k] = 0; 
    for (j = 0; j < nobs; j++)  
        if (info[j].photometric && info[j].selected && info[j].datsw[0]) 
            for (k = 0; k < 5; k++) 
                if (info[j].line_name[0] == johnson[k]) { 
                    photsw[k] = 1; 
                    n[k] += info[j].nspec; 
                }

    for (k = npt = 0; k < 5; k++) if (npt < n[k]) npt = n[k]; 
    phot =  matrix(0, 4, 0, npt-1); phi  = matrix(0, 4, 0, npt-1); 
    tmp1 =  vector(0,       npt-1); tmp2 = vector(0,       npt-1); 
    indx = ivector(0,       npt-1); 

    /* Copy containers into temporary matrices */
    for (k = 0; k < 5; k++) n[k] = 0; 
    for (j = p = 0; j < nobs; p += info[j].nspec, j++)  
        if (info[j].photometric && info[j].selected && info[j].datsw[0]) 
            for (k = 0; k < 5; k++) 
                if (info[j].line_name[0] == johnson[k]) {
                    for (nn = 0; nn < info[j].nspec; nn++) {
                        phi [k][n[k]+nn] = 0.5*(info[j].phasec + info[j].phasef); 
                        phot[k][n[k]+nn] = I[p+nn]; 
                    }
                    n[k] += info[j].nspec; 
                }

    /* Reorder matrices by increasing phases */
    for (k = 0; k < 5; k++) 
        if (photsw[k] == 1) { 
            indexx(n[k], phi[k], indx); 
            for (j = 0; j < n[k]; j++) {
                tmp1[j] = phi [k][indx[j]]; 
                tmp2[j] = phot[k][indx[j]]; 
            }
            for (j = 0; j < n[k]; j++) {
                phi [k][j] = tmp1[j]; 
                phot[k][j] = tmp2[j]; 
            }
        }

    /* Take phases at which the star is measured in all required bands */
    *nphot = 0; 
    for (k = 0; k < 5; k++) m[k] = 0; 
    do {
        max = -1.0e30; 
        for (k = 0; k < 5; k++) 
            if (photsw[k] == 1) 
                if (phi[k][m[k]] > max) max = phi[k][m[k]]; 

        for (k = nn = 0; k < 5; k++) 
            if (photsw[k] == 1) 
                if ((max - phi[k][m[k]]) > 0.0001) {
                    (m[k])++; 
                    nn++; 
                }

        if (!nn) {
            for (k = 0; k < 5; k++) 
                if (photsw[k] == 1) {
                    phi [k][*nphot] = phi [k][m[k]]; 
                    phot[k][*nphot] = phot[k][m[k]]; 
                    (m[k])++; 
                }
            (*nphot)++; 
        } 

        nn = 1; 
        for (k = 0; k < 5; k++) if (photsw[k] == 1) nn *= (m[k] < n[k]); 
    } while (nn == 1); 

    /* Save photometry and release temporary memory */
    *color  = matrix(0, 4, 0, *nphot-1); 
    *phases = vector(0, *nphot-1); 

    for (k = 0; k < 5; k++) 
        if (photsw[k] == 1) 
            for (j = 0; j < *nphot; j++) { 
jjj = phi[k][j]; if (phi[k][j] < 0.0) jjj--; 
                (*phases)[j] = phi[k][j]-jjj; 
                (*color)[k][j] = phot[k][j]; 
            }

    free_matrix(phot, 0, 4, 0, npt-1); free_matrix(phi,  0, 4, 0, npt-1); 
    free_vector(tmp1,       0, npt-1); free_vector(tmp2,       0, npt-1); 
    free_ivector(indx,      0, npt-1); 
}


static void indexx(int n, float *arrin, int *indx)
{
    int   j, l, ir, indxt, i;
    float q;

    for (j = 0; j < n; j++) indx[j] = j;
    l = n/2; ir = n-1;
    for (;;) {
        if (l > 0) {
            l--;
            indxt = indx[l];
            q = arrin[indxt];
        } else {
            indxt = indx[ir];
            q = arrin[indxt];
            indx[ir] = indx[0];
            ir--;
            if (ir == 0) {
                indx[0] = indxt;
                break;
            }
        }
        i = l; j = 2*l+1;
        while (j <= ir) {
            if (j < ir) if (arrin[indx[j]] < arrin[indx[j+1]]) j++;
            if (q < arrin[indx[j]]) {
                indx[i] = indx[j];
                i  = j;
                j += j+1;
            } else
                j = ir+1;
        }
        indx[i] = indxt;
    }
    return;
}


static void linear(float *x, float *y, int npt, float *lam, float *data, int nspec)
{
    int   k, xin;
    float slope;

    for (k = 0; k < npt; k++) {
        xin = locate(x[k], lam, nspec);
        slope = (x[k]-lam[xin]) / (lam[xin+1]-lam[xin]);
        y[k] = (data[xin+1]-data[xin])*slope + data[xin];
    }
}


/* Locates value in array */

static int locate(float x, float array[], int n)
{
    int klo, khi, k, dir;

    klo = 0;
    khi = n-1;
    if (array[0] < array[n-1])
        dir =  1;
    else
        dir = -1;
    while (khi-klo > 1) {
    k = (khi + klo)/2;
    if (((array[k] > x) && (dir == 1)) || ((array[k] < x) && (dir == -1)))
        khi = k;
    else
        klo = k;
    }
    return klo;
}


void gdyn(float **I, float **Q, float **U, float **V, spec_info_t **info, int nfile, 
          int *nobs, char *title)
{
    int   j, k, m, n, nx, ny, p, q, pp, pm, pmax, datsw_new[4], lut, *gg; 
    float **pt, laml, cent, tmp, min, max, resol, wid, *x, *y, *s, *xx, **im, 
          def, difp, difm, ymin, ymax, phase, step, xmin, xmax, vrad, vsin, ew, EW; 
float *mean; 

/* checks file compatibility and differentiates them if necessary */ 
    if (nfile > 2) { 
        printf("Gdyn only available with <= 2 input files\n"); 
        exit(1); 
    } else 
        for (k = n = pp = 0; k < nobs[0];  n += info[0][k].nspec, k++) 
            if (!info[0][k].photometric && info[0][k].selected) {  
                for (q = 0; q < 4; q++) datsw_new[q] = info[0][k].datsw[q];
                if (nfile == 2) { 
                    for (q = 0; q < 4; q++) datsw_new[q] *= info[1][k].datsw[q];
                    for (p = m = 0; p < nobs[1]; m += info[1][p].nspec, p++) 
                        if (!info[1][p].photometric && info[1][p].selected) 
                            if ((info[1][p].phasec == info[0][k].phasec) && 
                                (info[1][p].phasef == info[0][k].phasef)) break; 
                    if (p == nobs[1]) { 
                        printf("Observation at phase %.3f not found in second input file\n", 
                                0.5*(info[0][k].phasec+info[0][k].phasef));  
                        exit(1); 
                    } 
                    if ((info[1][p].nspec != info[0][k].nspec) || 
                    (fabs(info[1][p].laml-info[0][k].laml) > 0.1) || 
                    (fabs(info[1][p].lamu-info[0][k].lamu) > 0.1)) { 
                        printf("Spectra at phase %.3f incompatible\n", 
                                0.5*(info[0][k].phasec+info[0][k].phasef));  
                        exit(1); 
                    } 
                    for (q = 0; q < info[0][k].nspec; q++) { 
                        if (datsw_new[0]) I[0][n+q] -= I[1][m+q]; 
                        if (datsw_new[1]) Q[0][n+q] -= Q[1][m+q]; 
                        if (datsw_new[2]) U[0][n+q] -= U[1][m+q]; 
                        if (datsw_new[3]) V[0][n+q] -= V[1][m+q]; 
                    } 
                } 
                if (pp == 0) { 
                    step = (info[0][k].lamu-info[0][k].laml) / (info[0][k].nspec-1); 
                    cent = info[0][k].line_centre; 
                    laml = info[0][k].laml;  
                    nx   = info[0][k].nspec;  
                    pp = 1; 
                } 
            } 

/* gets user input */ 
    get_datsw(datsw_new, &n, 0); 
    for (k = m = 0; k < 4; k++) m += k * (datsw_new[k] == 1); 
    if      (m == 0) pt = I; 
    else if (m == 1) pt = Q; 
    else if (m == 2) pt = U; 
    else if (m == 3) pt = V; 

    printf("Phase min, max, resolution, specwidth : "); 
    scanf("%f %f %f %f", &min, &max, &resol, &wid); 
    printf("%f %f %f %f\n", min, max, resol, wid); 
    if (max < min) { tmp = min; min = max; max = tmp; }  
    ny = (float) (max-min)/resol; pmax = (float) 0.5*wid/resol; if (pmax < 1) pmax = 1;  

    EW = 0.0; 
    do {
        printf("Plot color (bw:0; brown:1, blue/red:2, green/red:3) : ");
        scanf("%d", &lut);
        if (lut == 0)      printf("bw\n");
        else if (lut == 1) printf("brown\n");
        else if (lut == 2) printf("blue/red\n");
        else if (lut == 3) printf("green/red\n");
    } while ((lut < 0) || (lut > 3));

    printf("Thresholds and default : ");
    scanf("%f %f %f", &ymin, &ymax, &def);
    printf("%f %f %f\n", ymin, ymax, def);

    printf("Radial and rotation velocities :"); 
    scanf("%f %f", &vrad, &vsin);
    printf("%f %f\n", vrad, vsin);


/* fills in the image */ 
    x = vector(0, nx-1); 
    y = vector(0, ny-1); 
    s = vector(0, nx-1); 
mean = vector(0, nx-1); 
for (j=0; j< nx; j++) mean[j] = 0.0; 

    im = matrix(0, nx-1, 0, ny-1); 
    for (j = 0; j < nx; j++) x[j] = 299792.5*((laml + step*j)/cent-1.0)+vrad; 
    for (j = 0; j < ny; j++) y[j] = min + (max-min)*j/(ny-1); 
    for (k = 0; k < nx; k++) for (j = 0; j < ny; j++) im[k][j] = def; 

    difp = 1.0e36; difm = -difp;  
    for (k = n = 0; k < nobs[0];  n += info[0][k].nspec, k++) 
        if (!info[0][k].photometric && info[0][k].selected) { 
            phase = 0.5*(info[0][k].phasec+info[0][k].phasef);  
while (phase < min) phase += 1.0; 
while (phase > max) phase -= 1.0; 
            if ((phase > min) && (phase < max)) { 
                xx = vector(0, info[0][k].nspec-1); 
                gg = ivector(0, info[0][k].nspec-1);
                step = (info[0][k].lamu-info[0][k].laml) / (info[0][k].nspec-1); 
                for (j = 0; j < info[0][k].nspec; j++) { 
                    xx[j] = 299792.5*((info[0][k].laml+step*j)/info[0][k].line_centre-1.0)+vrad; 
                    if (fabs(xx[j]-vrad) >= 380.0) gg[j] = 1;
                    else                      gg[j] = 0;
                } 
/*
                fit(xx,pt[0]+n,gg,info[0][k].nspec,0);
*/

ew = 0.0; for (j = 0; j < info[0][k].nspec; j++) 
if (gg[j] == 0) { 
ew += (pt[0][n+j]-info[0][k].continuum)*step; 
}
EW += ew/nobs[0];
printf("%f %f\n", phase, ew); 
                linear(x,s,nx,xx,pt[0]+n,info[0][k].nspec); 
for (j=0; j< nx; j++) mean[j] += s[j]/nobs[0]; 
                free_vector(xx, 0, info[0][k].nspec-1); 
                free_ivector(gg, 0, info[0][k].nspec-1);

                p = locate(phase, y, ny); 
                for (q = 0; q < nobs[0]; q++) 
                    if (!info[0][q].photometric && info[0][q].selected) { 
                        tmp = 0.5*(info[0][q].phasec+info[0][q].phasef);
                        if ((tmp-phase < difp) && (tmp-phase > 0.0)) difp = tmp-phase; 
                        if ((tmp-phase > difm) && (tmp-phase < 0.0)) difm = tmp-phase; 
                    } 
                if (difp >  10.0*resol*pmax) difp =  10.0*resol*pmax; 
                if (difm < -10.0*resol*pmax) difm = -10.0*resol*pmax; 
                pp = (float)  0.5*difp/resol+0.999; if (pp > pmax) pp = pmax; 
                pm = (float) -0.5*difm/resol+0.999; if (pm > pmax) pm = pmax; 
                for (j = 0; j < nx; j++) { 
                    for (q = p; (q <= p+pp) && (q < ny); q++) im[j][q] = s[j]; 
                    for (q = p; (q >= p-pm) && (q >= 0); q--) im[j][q] = s[j]; 
                } 
            } 
        } 
printf("%f\n", EW); 
printf("%f %f\n", x[0],x[1]); 
/*
save_ascfile(nx,1,x,mean,0,"mean","mean");
*/

    set_device(2); 
    tmp = max-min; min -= 0.02*tmp; max += 0.02*tmp;
    tmp = x[nx-1]-x[0]; xmin = x[0]-0.04*tmp; xmax = x[nx-1]+0.04*tmp; 
    init_plot(xmin,xmax,min,max,"Velocity (km/s)","Rotational phase", title,0,0);
/*
    init_plot(xmin,xmax,min,max,"Vitesse (km/s)","Fraction du cycle rotationnel", title,0,0);
*/
    image(im,x,y,nx,ny,ymin,ymax,20,1,lut,0);
    x[0] = x[1] = vrad; y[0] = min; y[1] = max; plot(x,y,2,1); 
    x[0] = x[1] = vrad-vsin; y[0] = min; y[1] = max; plot(x,y,2,1); 
    x[0] = x[1] = vrad+vsin; y[0] = min; y[1] = max; plot(x,y,2,1); 
    end_plot(); 


    free_vector(x, 0, nx-1); 
    free_vector(y, 0, ny-1); 
    free_vector(s, 0, nx-1); 
free_vector(mean, 0, nx-1); 
    free_matrix(im, 0, nx-1, 0, ny-1); 
}


static void fit(float *lam, float *spec, int *good, int nspec, int sub)
{
    float sx, sx2, sy, sxy, a, b, d, mean, ww, tmp;
    int   j, n, k;

    tmp = sx = sy = sx2 = sxy = 0.0;
    for (j = n = 0; j < nspec; j++)
        if (good[j]) {
            ww = 1.0;
            sx  += ww*lam[j];
            sy  += ww*spec[j];
            sx2 += ww*lam[j]*lam[j];
            sxy += ww*lam[j]*spec[j];
            tmp += ww;
        }
    d = tmp*sx2-sx*sx;
    a = (tmp*sxy-sx*sy)/d; b = (sx2*sy-sx*sxy)/d;

    if (sub) for (j = 0; j < nspec; j++) spec[j] -= a*lam[j]+b;
    else     for (j = 0; j < nspec; j++) spec[j] /= a*lam[j]+b;
}
