#include <stdio.h> 
#include <string.h>
#include "structures.h" 
#include "mem_alloc.h"
#include "file.h" 
#include "lsfit.h"
#include "plot.h" 

void get_spec_dims(spec_info_t spec_info[], int nobs, int *npns, int datsw[]); 
void get_data_to_scale(spec_info_t **spec_info, int nfile, int *nobs, int *photo, 
                      char name[]); 
void scale(float *I, float *Q, float *U, float *V, spec_info_t *info, int nobs); 


int main(void) 
{
    int    j, k, p, q, n, m, nfile, getname, open_ok, datsw[4], nobs, photo; 
    float  *I, *Q, *U, *V; 
    char   name[80], com[80]; 
    spec_info_t *info; 

    getname = 1; 

    /* Reads in the spectra */ 

        do { 
            open_ok = read_spec(&I,&Q,&U,&V,&nobs,&info,name,getname,com); 
            if (open_ok == 1) printf("File not found. "); 
        } while (open_ok == 1); 

    /* Plots the spectra */ 
    get_data_to_scale(&info, 1, &nobs, &photo, name); 
    if (!photo)  
        scale(I, Q, U, V, info, nobs); 

    getname = 1; 
    save_spec(I,Q,U,V,nobs,info,name,getname,com,1); 

    /* Release memory */ 
        get_spec_dims(info, nobs, &n, datsw); 
        if (datsw[0] == 1) free_vector(I, 0, n-1); 
        if (datsw[1] == 1) free_vector(Q, 0, n-1); 
        if (datsw[2] == 1) free_vector(U, 0, n-1); 
        if (datsw[3] == 1) free_vector(V, 0, n-1); 
        free_svector((void *) info); 
}


void get_data_to_scale(spec_info_t **spec_info, int nfile, int *nobs, int *photo, 
                      char name[]) 
{
    int    n,m,j,k,**flag,nlines,*type,*used; 
    char   **line_names, answer[2];

    flag = (int **) svector(nfile, (int) sizeof(int *)); 
    for (n = 0; n < nfile; n++) { 
        flag[n] = ivector(0, nobs[n]-1); 
        for (j = 0; j < nobs[n]; j++) flag[n][j] = 0; 
    }

    /* Count the number of different type of observation in all data files */ 
    nlines = 0; 
    for (n = 0; n < nfile; n++) 
        for (j = 0; j < nobs[n]; j++) 
            if (!flag[n][j]) {
                for (m = n; m < nfile; m++) 
                    for (k = j*(m==n); k < nobs[m]; k++) 
                        if (!strcmp(spec_info[m][k].line_name,spec_info[n][j].line_name)) 
                            flag[m][k] = 1;
                nlines++; 
            }

    type = ivector(0, nlines-1); 
    line_names = (char **) svector(nlines, (int) sizeof(char *)); 
    for (n = 0; n < nlines; n++) 
        line_names[n] = (char *) svector(80, (int) sizeof(char)); 

    /* Record the name and type of each type of observation */ 
    for (n = 0; n < nfile; n++) for (j = 0; j < nobs[n]; j++) flag[n][j] = 0; 
    nlines = 0; 
    for (n = 0; n < nfile; n++) 
        for (j = 0; j < nobs[n]; j++) 
            if (!flag[n][j]) {
                sprintf(line_names[nlines], "%s", spec_info[n][j].line_name); 
                type[nlines] = spec_info[n][j].photometric; 
                for (m = n; m < nfile; m++) 
                    for (k = j*(m==n); k < nobs[m]; k++) 
                        if (!strcmp(spec_info[m][k].line_name,spec_info[n][j].line_name))
                            flag[m][k] = 1;
                nlines++; 
            }

    /* Ask user for observation to scale */ 
    *photo = -1; 
    used = ivector(0, nlines-1); 
    if (nlines == 1) {
        used[0] = 1; 
        *photo = type[0]; 
        sprintf(name, "%s", line_names[0]); 
    } else { 
        for (k = 0; k < nlines; k++) used[k] = 0; 
        do {
            for (k = 0; k < nlines; k++) {
                if (type[k] == 1) continue; 
                printf("Do you want to scale the %s observations (y/n) : ",line_names[k]);
                scanf("%1s",answer);
                if ((answer[0] == 'Y') || (answer[0] == 'y')) 
                    if (*photo < 0) {
                        used[k] = 1; 
                        *photo = type[k]; 
                        sprintf(name, "%s", line_names[k]); 
                        if (*photo == 0) break; 
                    } else {
                        if (type[k] == 0) 
                            printf("You can't mix spectroscopy and photometry\n"); 
                        else 
                            used[k] = 1; 
                    }
            }
        } while (*photo < 0); 
    }

    /* Adjust spec_info */ 
    for (n = 0; n < nfile; n++) 
        for (j = 0; j < nobs[n]; j++) 
            for (k = spec_info[n][j].selected = 0; k < nlines; k++) 
                if (used[k] && !strcmp(spec_info[n][j].line_name,line_names[k])) 
                    spec_info[n][j].selected = 1; 

    for (n = 0; n < nfile; n++)  free_ivector(flag[n], 0, nobs[n]-1); 
    free_svector((void *) flag); 
    for (n = 0; n < nlines; n++) free_svector((void *) line_names[n]); 
    free_svector((void *) line_names); 
    free_ivector(type, 0, nlines-1); 
    free_ivector(used, 0, nlines-1); 
}


void get_spec_dims(spec_info_t spec_info[], int nobs, int *npns, int datsw[])
{
    int j,k;

    for (k = 0; k < 4; k++)
        datsw[k] = 0;

    *npns = 0;

    for (j = 0; j < nobs; j++) {
        *npns += spec_info[j].nspec;
        for (k = 0; k < 4; k++)
            if (spec_info[j].datsw[k]) datsw[k] = 1;
    }
}


void scale(float *I, float *Q, float *U, float *V, spec_info_t *info, int nobs)
{
    int   k, nph, n, j, p, deg; 
    float *ew, ewmean, *ewmod, *sn, step, ymin, ymax, xmin, xmax, tmp, maxdev; 
    double *b; 

    nph = 0; 
    for (k = 0; k < nobs; k++) if (!info[k].photometric && info[k].selected && info[k].datsw[0]) nph++; 
    if (nph < 2) return; 

    ewmean = 0.0; 
    sn = vector(0, nph-1); 
    ew = vector(0, nph-1); 
    for (k = n = p = 0; k < nobs; p += info[k].nspec, k++) 
        if (!info[k].photometric && info[k].selected && info[k].datsw[0]) {
            step = (info[k].lamu-info[k].laml)/(info[k].nspec-1); 

/*
            sn[n] = info[k].phasef - (int) info[k].phasef; 
            if (sn[n] < 0.0) sn[n] += 1.0; 
*/
            sn[n] = info[k].SN; 
            ew[n] = 0.0; 
            for (j = 0; j < info[k].nspec; j++) {
/*
                if ((info[k].laml+step*j < -18.0) || (info[k].laml+step*j > 18)) 
*/
                    ew[n] += step*(info[k].continuum-I[p+j]); 
/*
printf("%d %d %d %f %f %f %f %f\n", n, p, j, step,info[k].continuum,I[p+j],step*(info[k].continuum-I[p+j]),ew[n]); 
*/
            }
            ewmean += ew[n]/nph; 
            n++;  
        }

    maxdev = 3.0; deg = 2; 
    b = dvector(0,deg); 
    ewmod = vector(0,nph-1);  
    adjust(sn,ew,nph,b,deg,maxdev); 
    poly(sn,ewmod,nph,0.0,b,deg);
    free_dvector(b,0,deg); 

    set_device(3); 
    minmax(sn,nph,&xmin,&xmax); tmp = xmax-xmin; xmin -= 0.1*tmp; xmax += 0.1*tmp; 
    minmax(ew,nph,&ymin,&ymax); tmp = ymax-ymin; ymin -= 0.1*tmp; ymax += 0.1*tmp;     
    init_plot(xmin,xmax,ymin,ymax,"SN","EW"," ",0,0); 
    plot_sym(sn,ew,nph,4); 
    plot_sym(sn,ewmod,nph,1); 
    end_plot(); 

    for (k = n = p = 0; k < nobs; p += info[k].nspec, k++) {
        if (!info[k].photometric && info[k].selected  && info[k].datsw[0]) { 
            if (info[k].datsw[0]) 
                for (j = 0; j < info[k].nspec; j++) 
                    I[p+j] = info[k].continuum - ewmean/ew[n]*(info[k].continuum-I[p+j]); 
            if (info[k].datsw[3])  
                for (j = 0; j < info[k].nspec; j++) V[p+j] *= ewmean/ew[n]; 

            for (j = 0; j < 4; j++) info[k].datsel[j] = info[k].datsw[j]; 
            n++; 
        } else {
            info[k].selected = 1; 
            for (j = 0; j < 4; j++) info[k].datsel[j] = info[k].datsw[j]; 
        }
    }
    
    free_vector(sn, 0, nph-1); 
    free_vector(ew, 0, nph-1); 
    free_vector(ewmod, 0, nph-1);  
} 
