#include <stdio.h>
#include <math.h>
#include "constants.h"
#include "structures.h"
#include "mem_alloc.h"
#include "file.h"
#include "plot.h"
#include "grid.h"
#include "/sw/lib/pgplot/cpgplot.h"

void draw_field(float beta, float phi, float a); 
void draw_star(float inc, int rect, float phi0); 
void get_spec_dims(spec_info_t spec_info[], int nobs, int *npns, int datsw[]);
void draw_obs(spec_info_t *info, int nobs, int rect); 
void build_map(float *pt, int ngrid, float inc, float def, 
	       float ***im, float **x, float **y, float xmin, float xmax, 
               float ymin, float ymax, int *nx, 
	       int *ny, int rect, float phi0); 
void build_map2(float *Br, float *Bp, float *Bt, int ngrid, float inc, float def,
               float ***im, float ***im1, float ***im2, float **x, float **y, 
               int *nx, int *ny, float phi0); 


int main(int argc, char *argv[])
{
    int    i, j, n, getname, magnetic, nx, ny, open_ok, ngrid, rect, lut, nplot, npx, 
	   npy, nobs, datsw[4], npts, spec; 
    float  *x, *y, **im, **im1, **im2, *Cq, *Cm, *Br, *Bt, *Bp, inc, ve, max, *pt, def, ymin, ymax, 
	   *I, *Q, *U, *V, xmin, xmax, tmp, *zero, *diff, phi0; 
    char   filename[80], com[120], answer[2], title[80], labx[80]; 
    spec_info_t *info; 

    printf("Rectangular, polar, spherical or arrowed-spherical view (r/p/s/a) : "); 
    scanf("%1s", answer);
    rect = 0; 
    if ((answer[0] == 'r') || (answer[0] == 'R')) rect = 1;
    else if ((answer[0] == 's') || (answer[0] == 'S')) rect = -1;
    else if ((answer[0] == 'a') || (answer[0] == 'A')) rect = -2;
    if (rect == 1) printf("Rectangular\n");
    else if (rect == -1) printf("Spherical\n");
    else if (rect == -2) printf("Spherical with arrows\n");
    else           printf("Polar\n");

/*
if (rect == 1) set_device(5); 
else      set_device(4); 
*/
if (rect == 1) set_device(2); 
else      set_device(1); 
set_device(2); 

do { 
    printf("Number of plots, npx, npy : "); 
    scanf("%d %d %d", &nplot, &npx, &npy); 
    printf("%d %d %d\n", nplot, npx, npy); 
} while ((nplot > npx*npy)); 
init_multiplot(npx,npy); 

if (rect == 1)       cpgpap(0.0,0.45*npy/npx); 
else if (rect == -1) cpgpap(0.0,1.0*npy/npx); 
else                 cpgpap(0.0,0.8*npy/npx); 
/*
cpgpap(0.0,0.80*npy/npx); 
*/
cpgpap(10.0,1.2); 


for (n = 0; n < nplot; n++) { 
    getname = 1;
    do {
	open_ok = read_bright(&ngrid,&Cq,&Cm,&Br,&Bt,&Bp,filename,
			      getname,com,&inc,&ve,&magnetic);
	printf("%s\n", filename);
	if (open_ok == 1) printf("OOPS !!! File not found, try again.\n");
    } while (open_ok == 1);
    init_grid(ngrid, inc, ve);

    getname = spec = 1;
    open_ok = read_spec(&I,&Q,&U,&V,&nobs,&info,filename,getname,com);
    printf("%s\n", filename);
    if (open_ok == 1) { 
	printf("No spectrum found\n");
	spec = 0; 
    } 
    if (spec) { 
	get_spec_dims(info,nobs,&npts,datsw); 
	if (datsw[0] == 1) free_vector(I,0,npts-1);
	if (datsw[1] == 1) free_vector(Q,0,npts-1);
	if (datsw[2] == 1) free_vector(U,0,npts-1);
	if (datsw[3] == 1) free_vector(V,0,npts-1);
    } 

    def = 0.0; 
    if (magnetic) { 
        if (rect > -2) { 
	do {
	    def = 0.0; 
	    printf("Image quantity to plot (any of qmrpt) : ");
	    scanf("%1s", answer);
	    if (answer[0] == 'q') { 
		pt = Cq; 
		for (j = 0; j < ngrid; j++) Cq[j] = 1.0-Cq[j];  
//	for (j = 0; j < ngrid; j++) Cq[j] = 2.0*(1.0-Cq[j]);  
//def = 1.0; 
	    } else if (answer[0] == 'm') pt = Cm;
	    else if (answer[0] == 'r') pt = Br;
	    else if (answer[0] == 'p') pt = Bp;
	    else if (answer[0] == 't') pt = Bt;
	    else {
		printf("Choose any parameter among qmrpt\n");
		open_ok = 1;
	    }
	    printf("%1s\n", answer); 
	} while (open_ok);
        } 
    } else {
	pt = Cq; 
	for (j = 0; j < ngrid; j++) Cq[j] = 1.0-Cq[j];  
//	for (j = 0; j < ngrid; j++) Cq[j] = 2.0*(1.0-Cq[j]);  
//def = 1.0; 
    } 
//for (j = 0; j < ngrid; j++) Bp[j] = Br[j] = 0.0;  
//for (j = 0; j < ngrid; j++) Br[j] *= -1;  

    do {
	printf("Plot color (bw:0; brown:1, blue/red:2, green/red:3) : ");
	scanf("%d", &lut);
	if (lut == 0)      printf("bw\n");
	else if (lut == 1) printf("brown\n");
	else if (lut == 2) printf("blue/red\n");
	else if (lut == 3) printf("green/red\n");
    } while ((lut < 0) || (lut > 3));

    printf("Phase range : "); 
    scanf("%f %f", &xmin, &xmax); 
    printf("%f %f\n", xmin, xmax); 

    printf("Thresholds : "); 
    scanf("%f %f", &ymin, &ymax); 
    printf("%f %f\n", ymin, ymax); 
    if (lut == 0) def = ymax; 

    j = 0; 
    getchar();
    printf("Label on x axis : "); 
    while ((labx[j++] = getchar()) != '\n');
    labx[--j] = '\0';
    printf("%s\n", labx);

    j = 0; 
    printf("Title : "); 
    while ((title[j++] = getchar()) != '\n');
    title[--j] = '\0';
    printf("%s\n", title);

    if (rect <= -1) { 
        printf("Phase of display : "); 
        scanf("%f", &phi0); 
        printf("%f\n", phi0); 
    } 

    if ((rect == 0) || (rect == 1)) 
	if (inc > PI/6.0) inc = PI/6.0; 
    if (rect > -2) 
        build_map(pt,ngrid,inc,def,&im,&x,&y,xmin,xmax,ymin,ymax,&nx,&ny,rect,phi0); 
    else 
        build_map2(Br,Bp,Bt,ngrid,inc,def,&im,&im1,&im2,&x,&y,&nx,&ny,phi0); 

    if (rect == 1) { 
	max = TO_DEG(inc)+90.0; 
	tmp = xmax-xmin; xmin -= 0.015*tmp; xmax += 0.015*tmp; 
/*
	start_plot(1.015,-0.015,-TO_DEG(inc)*180.0-0.035*max,90.0+0.035*max,"Rotational phase","Latitude", title,0,0); 
*/
	start_plot(xmin,xmax,-TO_DEG(inc)-0.035*max,90.0+0.035*max,labx,"Latitude", title,0,0); 
	image(im,x,y,nx,ny,ymin,ymax,11,rect,lut,1);

	if (fabs(tmp) < 0.5) { 
	    zero = vector(0, ny-1); 
	    diff = vector(0, ny-1); 
	    for (j = 0; j < ny; j++) zero[j] = 0.0; 
	    for (j = 0; j < ny; j++) diff[j] = +0.012-0.037*cos(TO_RAD(y[j]))*cos(TO_RAD(y[j])); 
/*
	    for (j = 0; j < ny; j++) diff[j] = -0.01 + 0.1*sin(PI*y[j]/180.0)*sin(PI*y[j]/180.0); 
*/
	    plot(diff,y,ny,2); 
	    plot(zero,y,ny,1); 
	    free_vector(zero, 0, ny-1); 
	    free_vector(diff, 0, ny-1); 
	} 
    } else { 
/*
	start_plot(1.00,-1.20,1.00,-1.10," "," ",title,1,-1); 
	image(im,x,y,nx,ny,ymin,ymax,11,rect,lut,1); 
*/
	if (rect == -1) start_plot(1.00,-1.00,1.00,-1.00," "," "," ",1,-1); 
	else if (rect == -2) start_plot(1.20,-1.20,1.20,-1.20," "," "," ",1,-1); 
	else            start_plot(1.00,-1.10,1.00,-1.00," "," "," ",1,-1); 
//	else            start_plot(1.00,-1.20,1.00,-1.00," "," "," ",1,-1); 
	write_bigtext(title,0.0,-1.3); 
//	if (rect >= -1) image(im,x,y,nx,ny,ymin,ymax,11,rect,lut,(n==0)); 
	if (rect >= -1) image(im,x,y,nx,ny,ymin,ymax,11,rect,lut,1); 
	else            vectors(im,im1, im2,x,y,nx,ny,ymin,ymax,1.0,11); 
	
	draw_star(inc, rect, phi0); 
    } 
    if (spec) draw_obs(info,nobs,rect); 
/*
    draw_field(74.0,0.96,0.10); 
    draw_field(65.0,0.98,0.00); 
*/

    if (spec) free_svector((void *) info);
    free_vector(x, 0, nx-1); 
    free_vector(y, 0, ny-1); 
    free_matrix(im, 0, nx-1, 0, ny-1); 
    if (rect == -2) { 
        free_matrix(im1, 0, nx-1, 0, ny-1); 
        free_matrix(im2, 0, nx-1, 0, ny-1); 
    } 
    free_vector(Cq, 0, ngrid-1); 
    if (magnetic) { 
	free_vector(Cm, 0, ngrid-1); 
	free_vector(Br, 0, ngrid-1); 
	free_vector(Bp, 0, ngrid-1); 
	free_vector(Bt, 0, ngrid-1); 
    } 
}
for (n = nplot; n < npx*npy; n++) cpgpage(); 
end_plot(); 
}


void build_map(float *pt, int ngrid, float inc, float def, 
	       float ***im, float **x, float **y, float xmin, float xmax, 
               float ymin, float ymax, int *nx, 
	       int *ny, int rect, float phi0) 
{ 
    int   i, j, n; 
    float max, theta, phase, xx, yy, zz, temp, si, ci, ct, tmp; 

    if (ymax < ymin) {
        tmp = ymin; ymin = ymax; ymax = tmp; 
    }
    max = 0.5+inc/PI; 
    if (rect == 1) { 
	*nx = (float) 360.0*fabs(xmax-xmin); 
	*ny = (float) 180.0*max;  
    } else if (rect == 0) { 
	*nx = *ny = 200;   
    } else if (rect == -1) { 
	*nx = *ny = 200;   
    }
    

    *x  = vector(0, *nx-1); 
    *y  = vector(0, *ny-1); 
    *im = matrix(0, *nx-1, 0, *ny-1); 

    if (rect == 1) { 
	for (i = 0; i < *nx; i++) (*x)[i] = (float) xmin + (xmax-xmin)*(*nx-i-0.5) / *nx; 
	for (j = 0; j < *ny; j++) (*y)[j] = (float) 90.0 - 180.0*max*(j+0.5) / *ny; 
    } else { 
	for (i = 0; i < *nx; i++) (*x)[i] = (float) -2.0*(i+0.5) / *nx + 1.0; 
	for (j = 0; j < *ny; j++) (*y)[j] = (float) -2.0*(j+0.5) / *ny + 1.0; 
    } 

    for (i = 0; i < *nx; i++)
	for (j = 0; j < *ny; j++) {
	    (*im)[i][j] = def; 
	    if (rect == 1) { 
		theta = (*y)[j]; 
		phase = (*x)[i]; 
		if (phase < 0.0) phase += 1.0; 
		if (phase > 1.0) phase -= 1.0; 
	    } else if (rect == 0) { 
		theta = max * sqrt((*x)[i]*(*x)[i] + (*y)[j]*(*y)[j]); 
		if (theta >= max) continue;  

		theta = 90.0-theta*180.0; 
		if (theta == 90.0) 
		    phase = 0.0; 
		else if ((*y)[j] == 0.0) 
		    phase = 0.25 + 0.5*((*x)[i]<0.0); 
		else 
		    phase = 0.5*atan((*x)[i]/(*y)[j])/PI + 0.5*((*y)[j]<0.0); 
		if (phase < 0.0) phase += 1.0; 
/*
printf("here %d %d %f %f     %f %f\n", i, j, (*x)[i], (*y)[j], theta, phase); 
*/
	    } else if (rect == -1) { 
		yy = (*x)[i]; zz = -(*y)[j]; 
		temp = 1.0 - zz*zz - yy*yy;
		if (temp < 0.0) continue; 
		xx = sqrt(temp); 
		si = sin(inc); ci = cos(inc);
		theta = asin(xx*ci + zz*si);
/*
printf("here %d %d %f %f %f     %f %f %f %f\n", i, j, xx, yy, zz, temp, inc, si, ci); 
*/
		ct = cos(theta);
		if (ct == 0.0) phase = 0.0;
		else {
		    temp = yy/ct;
		    if (temp > 1.0)       temp = 1.0;
		    else if (temp < -1.0) temp = -1.0;
		    phase = 0.5*asin(temp)/PI;
		    if ((zz > 00) && (yy*yy+zz*zz/si/si > 1.0)) phase = 0.5-phase;
		}
		if (phase < 0.0) phase += 1.0; 
		phase += phi0; 
		theta *= 180.0/PI; 
	    } 
	    find_grid_loc(TO_RAD(theta),(1.0-phase)*2.0*PI,&n,0.0);
/*
printf("                               %f %f %d \n", theta, phase, n); 
*/
	    if ((n > 0) && (n <= ngrid)) {
                tmp = pt[--n]; 
                if (tmp < ymin) tmp = ymin; 
                if (tmp > ymax) tmp = ymax; 
		(*im)[i][j] = pt[n]; 
	    } else 
		(*im)[i][j] = def; 
	} 
}


void build_map2(float *Br, float *Bp, float *Bt, int ngrid, float inc, float def,
               float ***im, float ***im1, float ***im2, float **x, float **y, 
               int *nx, int *ny, float phi0)
{
    int   i, j, n, nst = 1000;
    float max, theta, phase, xx, yy, zz, temp, si, ci, ct, tmp, *B, *gamma, *chi, *th, *ph;
    cell_info_t *cell;
    cell_deriv_t *dr;

    xx = (float) ngrid/nst+0.4999; nst = xx;  if (nst < 1) nst = 1; 

        *nx = *ny = 201;

    *x  = vector(0, *nx-1);
    *y  = vector(0, *ny-1);
    *im = matrix(0, *nx-1, 0, *ny-1);
    *im1 = matrix(0, *nx-1, 0, *ny-1);
    *im2 = matrix(0, *nx-1, 0, *ny-1);

	for (i = 0; i < *nx; i++) (*x)[i] = (float) -2.0*(i+0.5) / *nx + 1.0; 
	for (j = 0; j < *ny; j++) (*y)[j] = (float) -2.0*(j+0.5) / *ny + 1.0; 

    cell  = (cell_info_t *) svector(ngrid,(int) sizeof(cell_info_t));
    dr    = (cell_deriv_t *) svector(ngrid,(int) sizeof(cell_deriv_t));
    B     = vector(0,ngrid-1);
    gamma = vector(0,ngrid-1);    
    chi   = vector(0,ngrid-1);    
    th = vector(0,ngrid-1);    
    ph = vector(0,ngrid-1);    

    for (i = 0; i < *nx; i++)
        for (j = 0; j < *ny; j++) 
            (*im)[i][j] = (*im1)[i][j] = (*im2)[i][j] = def;

    si = sin(inc); ci = cos(inc);
    geometry(phi0,ngrid,cell,1,Br,Bt,Bp,B,gamma,chi,dr);  

    for (i = 0; i < *nx; i++)
        for (j = 0; j < *ny; j++) {
            (*im)[i][j] = def;
            yy = (*x)[i]; zz = -(*y)[j];
            temp = 1.0 - zz*zz - yy*yy;
            if (temp < 0.0) continue;
            xx = sqrt(temp);
            theta = asin(xx*ci + zz*si);
            ct = cos(theta);
            if (ct == 0.0) phase = 0.0;
            else {
                temp = yy/ct;
                if (temp > 1.0)       temp = 1.0;
                else if (temp < -1.0) temp = -1.0;
                phase = 0.5*asin(temp)/PI;
                if ((zz > 00) && (yy*yy+zz*zz/si/si > 1.0)) phase = 0.5-phase;
            }
            if (phase < 0.0) phase += 1.0;
            phase += phi0;
            theta *= 180.0/PI;
            find_grid_loc(TO_RAD(theta),(1.0-phase)*2.0*PI,&n,0.0);
	    if ((n > 0) && (n <= ngrid)) {
                tmp = B[--n]; 
		(*im)[i][j] = tmp; 
	    } else 
		(*im)[i][j] = def; 
        } 

    locations(phi0,ngrid,th,ph); 
    for (n=0; n < ngrid; n += nst) 
        if (cell[n].mu > 0.1){  
            xx = sin(ph[n])*cos(th[n]); 
            yy = -ci*cos(ph[n])*cos(th[n]) + si*sin(th[n]); 
            i = (float) 0.5*(xx+1.0) * (*nx-1) + 0.5; 
            j = (float) 0.5*(yy+1.0) * (*ny-1) + 0.5; 
            if (Br[n] < 0.0) (*im)[i][j] *= -1.0; 
            (*im1)[i][j] = B[n]*sin(gamma[n]); 
            (*im2)[i][j] = chi[n]; 
//printf("%d %f %f    %f %f %d %d %f %f   %f %f %f   %f %f\n", n, TO_DEG(ph[n]), TO_DEG(th[n]),xx,yy,i,j,(*x)[i],(*y)[j],B[n],gamma[n]/PI*180.0,chi[n]/PI*180.0,(*im)[i][j], (*im2)[i][j]); 
//if (n > 50) return; 
        } 

    free_svector(cell);    
    free_vector(B,    0,ngrid-1);
    free_vector(gamma,0,ngrid-1);
    free_vector(chi,  0,ngrid-1);
    free_vector(th,  0,ngrid-1);
    free_vector(ph,  0,ngrid-1);
    free_svector(dr); 
}


void draw_star(float inc, int rect, float phi0)
{
    int j, k, nph, nst, nlt, npt, nlat, jj, nn; 
    float x[2], y[2], *xx, *yy, phi, tmp, lat, tt; 
    char  text[80]; 

    if (rect == 0) { 
    inc = 0.5+inc/PI; 
    for (j = -90; j <= 90; j += 30) {  
	tmp = (float) -j/180.0 + 0.5;  tmp /= inc;  
	if (tmp <= 1.0) {  
	    if (j == 0)          plot_circle(0.0,0.0,tmp,101); 
	    else if (tmp > 0.99) plot_circle(0.0,0.0,tmp,1); 
	    else                 plot_circle(0.0,0.0,tmp,2); 
	} 
    } 

    nst = 20;  
    for (j = 0; j < nst; j++) {
	x[0] =      sin(2.0*PI*j/nst); y[0] =      cos(2.0*PI*j/nst); 
	x[1] = 0.95*sin(2.0*PI*j/nst); y[1] = 0.95*cos(2.0*PI*j/nst); 
	plot(x,y,2,1); 
    } 
    nlt = 4;  
    for (j = 0; j < nlt; j++) {
	x[0] =      sin(2.0*PI*j/nlt); y[0] =      cos(2.0*PI*j/nlt); 
	x[1] = 0.90*sin(2.0*PI*j/nlt); y[1] = 0.90*cos(2.0*PI*j/nlt); 
	plot(x,y,2,101); 
	x[0] = 1.25*sin(2.0*PI*j/nlt); y[0] = 1.17*cos(2.0*PI*j/nlt)+0.03; 
	sprintf(text,"%.2f",(float) j/nlt); 
	write_text(text,x[0],y[0]); 
    } 
    } else if (rect <= -1) { 
	plot_circle(0.0,0.0,1.0,101); 
	npt = 1000; nph = 10; nlat = 7; 
	xx = vector(0, npt-1); 
	yy = vector(0, npt-1); 
	for (k = 0; k < nlat; k++) {
nn = 0; jj = -1; 
	    tmp = -0.5*PI + PI*k/(nlat-1); 
	    for (j = 0; j < npt; j++) { 
		phi = -PI + 2.0*PI*j/(npt-1); 
		tt = sin(inc)*cos(tmp)*cos(phi)+cos(inc)*sin(tmp);
		xx[j] = sin(phi)*cos(tmp); 
		yy[j] = cos(inc)*cos(tmp)*cos(phi)-sin(inc)*sin(tmp); 
if ((jj < 0) && (tt >= 0.0)) jj = j; 
if (tt >= 0.0) nn++; 
	    } 
	    if (k == nlat/2) plot(xx+jj,yy+jj,nn,1); 
	    else            plot(xx+jj,yy+jj,nn,2); 
	} 
	for (k = 0; k < nph; k++) {
nn = 0; jj = -1; 
	    phi = 2.0*PI*k/nph; 
            phi -= 2.0*PI*phi0; 
	    for (j = 0; j < npt; j++) { 
		tmp = -0.5*PI+PI*j/(npt-1); 
		tt = sin(inc)*cos(tmp)*cos(phi)+cos(inc)*sin(tmp);
		xx[j] = sin(phi)*cos(tmp); 
		yy[j] = cos(inc)*cos(tmp)*cos(phi)-sin(inc)*sin(tmp); 
if ((jj < 0) && (tt >= 0.0)) jj = j; 
if (tt >= 0.0) nn++; 
	    } 
	    if (k == 0) plot(xx+jj,yy+jj,nn,1); 
	    else        plot(xx+jj,yy+jj,nn,2); 
	} 
	free_vector(xx, 0, npt-1); 
	free_vector(yy, 0, npt-1); 
    } 
} 


void draw_obs(spec_info_t *info, int nobs, int rect)
{
    int j; 
    float x[2], y[2], phase; 

    cpgsclp(0); 
    for (j = 0; j < nobs; j++) 
	if (!info[j].photometric) {
	    phase = 0.5*(info[j].phasec+info[j].phasef); 
	    while (phase < 0.0) phase += 1.0; 
	    while (phase > 1.0) phase -= 1.0; 
	    if (rect) { 
		x[0] = phase; y[0] = 96; 
		x[1] = phase; y[1] = 100; 
		plot(x,y,2,1); 
	    } else { 
		x[0] = 1.05*sin(2.0*PI*phase); y[0] = 1.05*cos(2.0*PI*phase); 
		x[1] = 1.10*sin(2.0*PI*phase); y[1] = 1.10*cos(2.0*PI*phase); 
		plot(x,y,2,2); 
	    } 
	} 
    cpgsclp(1); 
}


void get_spec_dims(spec_info_t spec_info[], int nobs, int *npns, int datsw[])
{
    int j,k;

    for (k = 0; k < 4; k++)
	datsw[k] = 0;

    *npns = 0;

    for (j = 0; j < nobs; j++) {
	*npns += spec_info[j].nspec;
	for (k = 0; k < 4; k++)
	    if (spec_info[j].datsw[k]) datsw[k] = 1;
    }
}

void draw_field(float beta, float phi, float a)
{
int i, npt=360; 
float amp,ooa2,*xx,*yy,ca,sa,cb,sb,cp,sp,tmp,zz; 

ca = cos(2.0*PI*phi); 
sa = sin(2.0*PI*phi); 
cb = cos(2.0*PI*beta/360.0); 
sb = sin(2.0*PI*beta/360.0); 
ooa2=1.0/sqrt(1.0+a*a); 

amp = 90-beta-4; 
write_bigtext("+",(90-amp)/120.0*sa,(90-amp)/120.0*ca); 
write_bigtext("-",-(90+amp)/120.0*sa,-(90+amp)/120.0*ca); 
xx = vector(0,npt); 
yy = vector(0,npt); 
for (i = 0; i < npt; i++) 
{
phi = 2.0*PI*i/npt; 
cp = cos(phi); sp = sin(phi); 
xx[i] = ooa2*(ca*cp+sa*(cb*sp-a*sb)); 
yy[i] = ooa2*(-sa*cp+ca*(cb*sp-a*sb)); 
zz = ooa2*(-sb*sp-a*cb); 
printf("%d    %f %f %f\n", i,xx[i],yy[i],zz); 
amp = (90.0-180.0*asin(zz)/PI)/120.0; 
tmp = sqrt(xx[i]*xx[i]+yy[i]*yy[i]); 
if (amp/tmp > 1.0) {xx[i] *= 1.0/tmp; yy[i] *= 1.0/tmp; }
else if (tmp > 0.1) {xx[i] *= amp/tmp; yy[i] *= amp/tmp; }
}
plot(xx,yy,npt,2); 
free_vector(xx, 0, npt); 
free_vector(yy, 0, npt); 
} 
