#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "structures.h"
#include "constants.h"
#include "file.h"
#include "mem_alloc.h"
#include "noise.h"
#include "utilities.h"


int main(argc,argv)
int argc;
char *argv[];
{
    int    j,k,sp,err,batch,nobs,seed,dd,ni,nd,nphot,nspec;
    float  *Im,*Qm,*Um,*Vm,dummy,*pt; 
    spec_info_t *spec_info;
    double fp,fm,moy,ii,vv,ro,pm,SN,SNnew,tmp,tmpi,tmpv,tmp2,tmpp;
    char   filename[80],com[120],answer[2];

    parse_args(argc,argv,&batch,&dummy);

    do {
        err = read_spec(&Im,&Qm,&Um,&Vm,&nobs,&spec_info,filename,
                            1,com);
        if (batch) printf("%s\n",filename);
        if (err)
            printf("File not found.  Try again\n");
    } while (err);

    printf("Do you want to change the noise level in the data (y/n) : ");
    scanf("%1s",answer);
    if ((answer[0] == 'Y') || (answer[0] == 'y')) {
	if (batch) printf("yes\n");
        printf("Input S/N value, RO noise, photom. acc. (mmag) and seed : "); 
        scanf("%lf %lf %lf %d",&SNnew,&ro,&pm,&seed);
        if (batch) printf("%g %g %g %d\n",SNnew,ro,pm,seed);
    } else {
	if (batch) printf("no\n");
	SNnew = -1;  ro = 0.0;
        printf("Input the random number seed : "); 
        scanf("%d",&seed);
        if (batch) printf("%d\n",seed);
    }
    
    srand((unsigned int) seed);

    nspec = nphot = 0; 
    for (j = 0; j < nobs; j++) 
	if (!spec_info[j].photometric) nspec += spec_info[j].nspec; 
        else                           nphot += spec_info[j].nspec; 
printf("Spec pts: %d, Phot pts: %d\n", nspec, nphot); 

    sp = 0;  tmpi = tmpv = tmpp = 0.0; 
    for (j = 0; j < nobs; j++) {
	spec_info[j].selected = 1;

        for (dd = 0; dd < 4; dd++)  
            spec_info[j].datsel[dd] = spec_info[j].datsw[dd];

	if (!spec_info[j].photometric) {
	    if (SNnew > 0) {
	        SN = SNnew;
	        spec_info[j].SN = SN;
	    } else
	        SN = spec_info[j].SN;
            moy = SN*SN/spec_info[j].continuum;

nd = ni = 0; tmp2 = 0.0; 
for (dd = 1; dd < 4; dd++) { 
    if (dd == 1) pt = Qm; 
    else if (dd == 2) pt = Um; 
    else if (dd == 3) pt = Vm; 

    if (spec_info[j].datsw[0])  ni++; 
    if (spec_info[j].datsw[dd]) nd++; 

            for (k = 0; k < spec_info[j].nspec; k++)  { 
                if (spec_info[j].datsw[0])  ii = Im[sp+k];
	        else                        ii = spec_info[j].continuum;

                if (spec_info[j].datsw[dd]) vv = pt[sp+k]; 
                else                        vv = 0.0; 

                if (SN <= 1000.0) { 
                    fp = poidev(0.5*moy*(ii+vv)); 
                    fm = poidev(0.5*moy*(ii-vv)); 
                } else { 
                    fp = 0.5*moy*(ii+vv);  fm = 0.5*moy*(ii-vv); 
                    fp += sqrt(fp)*gasdev(); 
                    fm += sqrt(fm)*gasdev(); 
                } 
                fp += ro*gasdev(); 
                fm += ro*gasdev(); 
                if ((spec_info[j].datsw[0]) && (ni == 1)) { 
                    Im[sp+k] = (fp+fm)/moy; 
                    tmp = Im[sp+k]-ii; 
                    tmpi += tmp*tmp/nspec; 
                } 
                if (spec_info[j].datsw[dd]) {
                    pt[sp+k] = (fp-fm)/moy; 
                    tmp = pt[sp+k]-vv; 
                    tmp2 += tmp*tmp/nspec; 
                } 
            }
}

if (nd) tmpv += tmp2/nd; 
/*
printf(">>> %d %f\n", nd, 1.0/sqrt(tmp2/nd*nobs)); 
*/

	} else {
	    if (SNnew > 0) {
		SN = 0.001*pm;
	        spec_info[j].SN = SN;
	    } else
	        SN = spec_info[j].SN;
            for (k = 0; k < spec_info[j].nspec; k++) 
                if (spec_info[j].datsw[0]) { 
                    tmp = SN*gasdev();
                    Im[sp+k] += tmp; 
                    tmpp += tmp*tmp/nphot; 
                } 
	}
        sp += spec_info[j].nspec; 
    }

    if (tmpi) printf("SNi = %.2f\n", tmpi = 1.0/sqrt(tmpi)); 
    if (tmpv) printf("SNp = %.2f\n", tmpv = 1.0/sqrt(tmpv)); 
    if (tmpp) printf("pht = %f\n", sqrt(tmpp)); 
/*
SN = (tmpi+tmpv)/((tmpi!=0.0)+(tmpv!=0.0)); 
for (j = 0; j < nobs; j++) if (!spec_info[j].photometric) spec_info[j].SN = SN;
*/
    save_spec(Im,Qm,Um,Vm,nobs,spec_info,filename,1,com,1);
    if (batch) printf("%s\n",filename);
}


static double poidev(xm)
double xm;
{
    static float sq,alxm,g,oldm = -1.0; 
    float em,t,y; 

    if (xm < 12.0) {
        if (xm != oldm) {
            oldm = xm; 
            g = exp(-xm); 
         }
         em = -1.0; t = 1.0; 
         do {
             em++; 
             t *= (double) rand()/(RAND_MAX+1.0); 
         } while (t > g); 
    } else {
        if (xm != oldm) {
            oldm = xm; 
            sq = sqrt(2.0*xm); 
            alxm = log(xm); 
            g = xm*alxm - gammln(xm+1.0); 
        }
        do {
            do {
                y = tan(PI*rand()/(RAND_MAX+1.0)); 
                em = sq*y + xm; 
            } while (em < 0.0); 
            em = floor(em); 
            t = 0.9*(1.0+y*y)*exp(em*alxm-gammln(em+1.0)-g); 
        } while ((double) rand()/(RAND_MAX+1.0) > t); 
    }
    return em; 
}


static double gammln(xx)
double xx;
{
    double cof[6],stp,x,tmp,ser;
    double  gam; 
    int    j; 

    cof[0] =  76.180091730000; cof[1] = -86.50532033000; 
    cof[2] =  24.014098220000; cof[3] =  -1.23173951600;
    cof[4] =   0.120858003e-2; cof[5] =  -0.53638200e-5;
    stp = 2.50662827465; 
    x   = (double) -1.0 + xx; 
    tmp = x + 5.50; tmp = (x+0.50)*log(tmp)-tmp; 
    ser = 1.0; 
    for (j = 0; j < 6; j++) {
        x   += 1.0; 
        ser += cof[j]/x; 
    }
    gam = (double) tmp + log(stp*ser); 
    return gam; 
}


static float gasdev()
{
    static int iset = 0;
    static float gset;
    float fac,rsq,v1,v2;

    if (iset == 0) {
	do {
	    v1 = 2.0*rand()/(RAND_MAX+1.0) - 1.0;
	    v2 = 2.0*rand()/(RAND_MAX+1.0) - 1.0;
	    rsq  = v1*v1 + v2*v2;
        } while (rsq >= 1.0 || rsq == 0.0);
	fac = sqrt(-2.0*log(rsq)/rsq);
	gset = v1*fac;
	iset = 1;
	return v2*fac;
    } else {
	iset = 0;
	return gset;
    }
}
