#include <stdio.h>
#include <math.h>
#include "mem_alloc.h"
#include "structures.h"

void minmax(float spec[], int length, float *min, float *max); 
double gammp(double a, double x);
static void nrerror(char error_text[]); 
static void gcf(double *gammcf, double a, double x, double *gln);
static void gser(double *gamser, double a, double x, double *gln);
static double gammln(double xx); 
void get_spec_dims(spec_info_t spec_info[], int nobs, int *npns, int datsw[]); 


int main(void)
{
    int    getname, open_ok, nspec, nfile, n, m, j, k, ntot, bin, kk, nn, mm, n1, n2; 
    float  *I, *lam, *errI, ymin, ymax, threshold, chi1, chi2, norm2, norm1, mean1, mean2, 
           ww, liml, limu, stepvel; 
    char   filename[80], comment[120];
    double prob1, prob2; 

    getname = 1;

    /* Reads in the spectra */
    do {
        open_ok = read_ascfile(&nspec, &nfile, &lam, &I, getname, filename, comment);
        if (open_ok == 1) printf("File not found. ");
    } while (open_ok == 1);
    errI = I+nspec; 
/*
    threshold = 0.30; 
    minmax(I, nspec, &ymin, &ymax);
    liml = lam[0]; limu = lam[nspec-1];
    for (k = 0; k < nspec; k++)
        if (I[k] < ymax-threshold*(ymax-ymin)) {
            if (liml > lam[k]) liml = lam[k];
            if (limu < lam[k]) limu = lam[k];
        }
    stepvel = (lam[nspec-1]-lam[0])/(nspec-1);
    liml -= 5.0*stepvel; if (liml < lam[0])       liml = lam[0];
    limu += 5.0*stepvel; if (limu > lam[nspec-1]) limu = lam[nspec-1];
*/
    liml = -118.0; 
    limu =  132.0; 

    chi1 = chi2 = mean1 = mean2 = norm1 = norm2 = 0.0; n1 = n2 = 0; 
    for (k = 0; k < nspec; k++) {
printf("%d %f\n", k, lam[k]); 
        ww = 1.0/errI[k]; ww *= ww; 
        if ((lam[k] < liml) || (lam[k] > limu)) { 
            mean1 += I[k]*ww; 
            chi1 += I[k]*I[k]*ww; 
            norm1 += ww; 
            n1++; 
        } else { 
            mean2 += I[k]*ww; 
            chi2 += I[k]*I[k]*ww; 
            norm2 += ww; 
            n2++; 
        } 
    } 
printf("here %d %d\n", n1, n2); 
    chi1 = chi1 - mean1*mean1/norm1;  
    chi2 = chi2 - mean2*mean2/norm2; 
    prob1 = gammp((double) 0.25*n1, 0.5*chi1);
    prob2 = gammp((double) 0.25*n2, 0.5*chi2);

    printf("   Outside stellar lines: %e %e %e %d\n", chi1, prob1, 1.0-prob1, n1);
    printf("    Inside stellar lines: %e %e %e %d\n", chi2, prob2, 1.0-prob2, n2);

    free_vector(lam, 0, nspec-1); 
    free_vector(I, 0, nfile*nspec-1); 

}



#define ITMAX 100
#define EPS 3.0e-7
#define FPMIN 1.0e-30

double gammp(double a, double x)
{
        double gamser,gammcf,gln;

        if (x < 0.0 || a <= 0.0) nrerror("Invalid arguments in routine gammp");
        if (x < (a+1.0)) {
                gser(&gamser,a,x,&gln);
                return gamser;
        } else {
                gcf(&gammcf,a,x,&gln);
                return 1.0-gammcf;
        }
}


static void gcf(double *gammcf, double a, double x, double *gln)
{
	int i;
	double an,b,c,d,del,h;

	*gln=gammln(a);
	b=x+1.0-a;
	c=1.0/FPMIN;
	d=1.0/b;
	h=d;
	for (i=1;i<=ITMAX;i++) {
		an = -i*(i-a);
		b += 2.0;
		d=an*d+b;
		if (fabs(d) < FPMIN) d=FPMIN;
		c=b+an/c;
		if (fabs(c) < FPMIN) c=FPMIN;
		d=1.0/d;
		del=d*c;
		h *= del;
		if (fabs(del-1.0) < EPS) break;
	}
	if (i > ITMAX) nrerror("a too large, ITMAX too small in gcf");
	*gammcf=exp(-x+a*log(x)-(*gln))*h;
}


static void gser(double *gamser, double a, double x, double *gln)
{
	int n;
	double sum,del,ap;

	*gln=gammln(a);
	if (x <= 0.0) {
		if (x < 0.0) nrerror("x less than 0 in routine gser");
		*gamser=0.0;
		return;
	} else {
		ap=a;
		del=sum=1.0/a;
		for (n=1;n<=ITMAX;n++) {
			++ap;
			del *= x/ap;
			sum += del;
			if (fabs(del) < fabs(sum)*EPS) {
				*gamser=sum*exp(-x+a*log(x)-(*gln));
				return;
			}
		}
		nrerror("a too large, ITMAX too small in routine gser");
		return;
	}
}


static double gammln(double xx)
{
	double x,y,tmp,ser;
	static double cof[6]={76.18009172947146,-86.50532032941677,
		24.01409824083091,-1.231739572450155,
		0.1208650973866179e-2,-0.5395239384953e-5};
	int j;

	y=x=xx;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.000000000190015;
	for (j=0;j<=5;j++) ser += cof[j]/++y;
	return -tmp+log(2.5066282746310005*ser/x);
}

static void nrerror(char error_text[])
{
    printf("%s\n", error_text); 
    exit(1); 
}
#undef ITMAX
#undef EPS

void get_spec_dims(spec_info_t spec_info[], int nobs, int *npns, int datsw[])
{
    int j,k;

    for (k = 0; k < 4; k++)
        datsw[k] = 0;

    *npns = 0;

    for (j = 0; j < nobs; j++) {
        *npns += spec_info[j].nspec;
        for (k = 0; k < 4; k++)
            if (spec_info[j].datsw[k]) datsw[k] = 1;
    }
}


#define BIG 1.0e30
void minmax(float spec[], int length, float *min, float *max)
{
    int j;

    *min = BIG; *max = -BIG;
    for (j = 0; j < length; j++) {
        if (*min > spec[j]) *min = spec[j];
        if (*max < spec[j]) *max = spec[j];
    }
}
#undef BIG

