/*

This package contains the subroutines used to manipulate the data into the
forms required by the MEM package.
*/

#include <math.h>
#define COMPILING_RESPONSE
#include "structures.h"
#include "constants.h"
#include "grid.h"
#include "line.h"
#include "mem_alloc.h"
#include "response.h"
#include "spectrum.h"
#include "smear.h"


static float mult, fac, upf; 

void response(float **R, float Cq[], float Cm[], float Br[], float Bt[], float Bp[],float lev,
              float I[], float Q[], float U[], float V[],
              int ngrid, int nobs, spec_info_t spec_info[], spec_sample_t sampling,
              int imsw[], int magnetic, int calc_all, int calc_quasi)
{
    int   i,j,k,l,ii,dd,imin,datin,i1,k1,npns,ns_max,nps,dp;
    float *B,*gamma,*chi,*II,*QQ,*UU,*VV,***dObs,***dSks;
    float area,phase,dph,weight,*cont,c0;
    static float dummy=0.0;
    cell_info_t *cell;
    cell_deriv_t *dr;

    npns = ns_max = 0;
    for (j = 0; j < nobs; j++) {
        npns += spec_info[j].nspec;  /* The total number of spectral points */
        if (spec_info[j].nspec > ns_max)  ns_max = spec_info[j].nspec;
    }

    cell  = (cell_info_t *) svector(ngrid,(int) sizeof(cell_info_t));      
    B     = vector(0,ngrid-1);    
    gamma = vector(0,ngrid-1);    
    chi   = vector(0,ngrid-1);    
    dr    = (cell_deriv_t *) svector(ngrid,(int) sizeof(cell_deriv_t));      
    II    = vector(0,ns_max-1);
    QQ    = vector(0,ns_max-1);
    UU    = vector(0,ns_max-1);
    VV    = vector(0,ns_max-1);
    dObs  = matrix3D(0,3,0,4,0,ns_max-1);
    dSks  = matrix3D(0,3,0,4,0,ns_max-1);

    if (imsw[0] == 2) { 
        cont = vector(0, nobs-1); 
        contin(nobs,spec_info,cont,ngrid,Cq,Cm,magnetic); 
    } 
    mult = 0.01/ngrid; fac = mult*lev; 

    nps = 0;  /*  Marks where we are along the full spectral record  */
    dp = 0;   /* Marks where we are in the fitted data being placed in the response matrix */
    for (j = 0; j < nobs; j++) {
	if (spec_info[j].selected) {
	    if (calc_all || !spec_info[j].linear || (calc_quasi && spec_info[j].linear == 2)) {
                if (imsw[0] == 2)      c0 = cont[j]*fac;
                else if (imsw[0] == 3) c0 = fac;
                else                   c0 = 0.0; 

                  /* Zero the accumulated spectrum arrays */
                if (spec_info[j].datsel[0])
                    for (k = 0; k < spec_info[j].nspec; k++)  I[nps+k] = c0;
                if (spec_info[j].datsel[1])
                    for (k = 0; k < spec_info[j].nspec; k++)  Q[nps+k] = 0.0;
                if (spec_info[j].datsel[2])
                    for (k = 0; k < spec_info[j].nspec; k++)  U[nps+k] = 0.0;
                if (spec_info[j].datsel[3])
                    for (k = 0; k < spec_info[j].nspec; k++)  V[nps+k] = 0.0;

                  /* Zero the response matrix */
                i1 = 0;
                for (ii = 0; ii < 5; ii++) {
                    if (imsw[ii]) {
                        datin = 0;
                        for (dd = 0; dd < 4; dd++) {
                            if (spec_info[j].datsel[dd]) {
                                k1 = datin*spec_info[j].nspec + dp;
                                for (k = 0; k < spec_info[j].nspec; k++)  
				     for (i = 0; i < ngrid; i++) R[k1+k][i1+i] = 0.0;
                                datin++;
                            }
                        }
                        i1 += ngrid; 
                    }
                }

	        dph  = (spec_info[j].phasef - spec_info[j].phasec)/(spec_info[j].nsample+1);
	        weight = 1.0 / (float) spec_info[j].nsample;
	        for (l = 1; l <= spec_info[j].nsample; l++) {
	            phase  = spec_info[j].phasec + l*dph;

                    geometry(phase,ngrid,cell,magnetic,Br,Bt,Bp,B,gamma,chi,dr);
                    for (i = 0; i < ngrid; i++) {
                        area = cell[i].parea*weight;
                        if (area > 0.0) {  /* The cell is visible */

			      /*  Calculate the local spectrum and derivatives */
                            if (magnetic)
                                doppl_deriv(dSks,II,QQ,UU,VV,spec_info[j],sampling,
                                    Cq[i],Cm[i],B[i],gamma[i],chi[i],cell[i].mu,cell[i].vel,
                                    imsw,magnetic);
                            else
                                doppl_deriv(dSks,II,QQ,UU,VV,spec_info[j],sampling,
                                    Cq[i],dummy,dummy,dummy,dummy,cell[i].mu,cell[i].vel,
                                    imsw,magnetic);

                              /* The arrays |II|, |QQ|, |UU| and |VV| are multiplied by the projected area
                                 of the grid cell then added to the accumulated spectrum. */
                            if (spec_info[j].datsel[0])
                                for (k = 0; k < spec_info[j].nspec; k++)  I[nps+k] += area*II[k];
                            if (spec_info[j].datsel[1])
                                for (k = 0; k < spec_info[j].nspec; k++)  Q[nps+k] += area*QQ[k];
                            if (spec_info[j].datsel[2])
                                for (k = 0; k < spec_info[j].nspec; k++)  U[nps+k] += area*UU[k];
                            if (spec_info[j].datsel[3])
                                for (k = 0; k < spec_info[j].nspec; k++)  V[nps+k] += area*VV[k];

                              /* Transform the derivatives from the Stokes frame to the stellar frame */
                            if (magnetic) {
                                for (dd = 0 ; dd < 4; dd++)
                                    if (spec_info[j].datsel[dd]) {
#pragma _CNX no_recurrence
                                        for (k = 0; k < spec_info[j].nspec; k++) {
                                            dObs[dd][0][k] = dSks[dd][0][k];  /* $\partial S / \partial C_q$ */
                                            dObs[dd][1][k] = dSks[dd][1][k];  /* $\partial S / \partial C_m$ */
                                            if (B[i] != 0.0) {
                                                dObs[dd][2][k] = dSks[dd][2][k]*Br[i]/B[i]
                                                               + dSks[dd][3][k]*dr[i].dcgdBr
                                                               + dSks[dd][4][k]*dr[i].dchdBr; /* $\partial S / \partial B_r$ */
                                                dObs[dd][3][k] = dSks[dd][2][k]*Bt[i]/B[i]
                                                               + dSks[dd][3][k]*dr[i].dcgdBt
                                                               + dSks[dd][4][k]*dr[i].dchdBt; /* $\partial S / \partial B_t$ */
                                                dObs[dd][4][k] = dSks[dd][2][k]*Bp[i]/B[i]
                                                               + dSks[dd][3][k]*dr[i].dcgdBp
                                                               + dSks[dd][4][k]*dr[i].dchdBp; /* $\partial S / \partial B_p$ */
                                            } else {
                                                dObs[dd][2][k] = dObs[dd][3][k] = dObs[dd][4][k] = dSks[dd][2][k];
                                            }
                                        }
                                    }
                            } else  
                                for (k = 0; k < spec_info[j].nspec; k++) 
                                     dObs[0][0][k] = dSks[0][0][k];  /* $\partial I / \partial C_q$ */

                              /* Copy the derivatives into the response matrix */
                            i1 = 0;
                            for (ii = 0; ii < 5; ii++) {
                                if (imsw[ii]) {
                                    datin = 0;
                                    for (dd = 0; dd < 4; dd++) {
                                        if (spec_info[j].datsel[dd]) {
                                            k1 = datin*spec_info[j].nspec + dp;
                                            for (k = 0; k < spec_info[j].nspec; k++)
                                                R[k1+k][i1+i] += area*dObs[dd][ii][k];
                                            datin++;
                                        }
                                    }
                                    i1 += ngrid; 
                                }
                            }
                        }
                    }
                }
	    }
	    for (dd = 0; dd < 4; dd++)
		if (spec_info[j].datsel[dd])
                   dp += spec_info[j].nspec;
        }
        nps += spec_info[j].nspec;
    }

    if (imsw[0] == 2) free_vector(cont,0,nobs-1); 
    free_svector(cell);    
    free_vector(B,    0,ngrid-1);    
    free_vector(gamma,0,ngrid-1);  
    free_vector(chi,  0,ngrid-1);      
    free_svector(dr);
    free_vector(II,   0,ns_max-1);
    free_vector(QQ,   0,ns_max-1);
    free_vector(UU,   0,ns_max-1);
    free_vector(VV,   0,ns_max-1);
    free_matrix3D(dObs,0,3,0,4,0,ns_max-1);
    free_matrix3D(dSks,0,3,0,4,0,ns_max-1);
}


/*
The following routine doppler_deriv() uses the spectral line routine
to evaluate the derivatives at a given phase and point on the stellar surface.
*/

static void doppl_deriv(float ***dSks, float I[], float Q[], float U[],
                        float V[], spec_info_t spec_info, spec_sample_t sampling, double Cq,
                        double Cm, double B, double gamma, double chi, double mu, double vel,
                        int imsw[], int magnetic)
{
    int    j,k,n,ii,dd,os,nspec,mod_imsw[5];
    float  **Sp,***Dr,*tmp,chii;
    double delta,*lambda,l0,shift,temp,zeta_rt,rad_mt,tan_mt;

    if (!magnetic)  B = 0.0;
    l0      = find_line_centre(spec_info.line_id);
    zeta_rt = find_macro_turb(spec_info.line_id);
    temp   = zeta_rt*l0*TwoSqrtLog2/C_l;
    rad_mt = temp*mu; 
    tan_mt = temp*sqrt(1.0-mu*mu);

    chii = chi + spec_info.polariser;

    nspec = spec_info.nspec;
    if (!spec_info.photometric) {
        os = sampling.oversample;
        n = os*(nspec-1) + 1;      /* Why ? Check this !!! */
    } else {
        os = 1;  /* No oversampling for photometry */
        n = nspec;
    }

    if (os == 1) {  /* No oversampling */
        Sp = pfvector(0,3);
        Sp[0] = I;  Sp[1] = Q;  Sp[2] = U;  Sp[3] = V;
        Dr = dSks;
    } else { /* Oversampling */
        Sp = matrix(0,3,0,n-1);
        Dr = matrix3D(0,3,0,4,0,n-1);
    }

    if (n > 1)
	delta = (spec_info.lamu-spec_info.laml)/(double)(n-1);
    else
	delta = 1.0 + 1000.0*spec_info.FWHM;  /* Disables the instrumental broadening */
    shift = (1.0 - vel/C_l);

    lambda = dvector(0,n-1);
#pragma _CNX no_recurrence
    for (j = 0; j < n; j++)
        lambda[j] = (j*delta + spec_info.laml)*shift;
    
    line_deriv(spec_info.line_id,Dr[0],Dr[1],Dr[2],Dr[3],
	       &(Sp[0][0]),&(Sp[1][0]),&(Sp[2][0]),&(Sp[3][0]),
	       n,lambda,Cq,Cm,mu,B,gamma,chii,magnetic);

    free_dvector(lambda,0,n-1);

    find_derivs_to_smear(imsw,mod_imsw,magnetic);

    tmp   = vector(0,n-1);
    for (dd = 0; dd < 4; dd++) {
        if (spec_info.datsel[dd]) {

	    if (!spec_info.broadened)
                smear(spec_info.FWHM,n,Sp[dd],delta);
            for (j = 0; j < n; j++) tmp[j] = Sp[dd][j];
            smear(rad_mt,n,Sp[dd],delta);
            smear(tan_mt,n,tmp,delta);
            for (j = 0; j < n; j++) Sp[dd][j] = 0.5*(Sp[dd][j] + tmp[j]);

            for (ii = 0; ii < 5; ii++)
                if (mod_imsw[ii]) {

	            if (!spec_info.broadened)
                        smear(spec_info.FWHM,n,Dr[dd][ii],delta);
                    for (j = 0; j < n; j++) tmp[j] = Dr[dd][ii][j];
                    smear(rad_mt,n,Dr[dd][ii],delta);
                    smear(tan_mt,n,tmp,delta);
                    for (j = 0; j < n; j++) Dr[dd][ii][j] = 0.5*(Dr[dd][ii][j] + tmp[j]);

                }
        }
    }
    free_vector(tmp,0,n-1);

    if (os == 1) {  /* No oversampling */
        free_pfvector(Sp,0,3);
    } else { /* Oversampling */
        if (spec_info.datsel[0])
            for (j = 0, k = 0; j < nspec; j++, k+=os) I[j] = Sp[0][k];
        if (spec_info.datsel[1])
            for (j = 0, k = 0; j < nspec; j++, k+=os) Q[j] = Sp[1][k];
        if (spec_info.datsel[2])
            for (j = 0, k = 0; j < nspec; j++, k+=os) U[j] = Sp[2][k];
        if (spec_info.datsel[3])
            for (j = 0, k = 0; j < nspec; j++, k+=os) V[j] = Sp[3][k];
        free_matrix(Sp,0,3,0,n-1);

        for (dd = 0; dd < 4; dd++)
            if (spec_info.datsel[dd])
                for (ii = 0; ii < 5; ii++)
                    for (j = 0, k = 0; j < nspec; j++, k+=os)
                        dSks[dd][ii][j] = Dr[dd][ii][k];
        free_matrix3D(Dr,0,3,0,4,0,n-1);
    }
}


static void find_derivs_to_smear(int imsw[], int mod_imsw[], int magnetic)
{
    int ii;

    if (magnetic) {
	mod_imsw[0] = imsw[0];
	mod_imsw[1] = imsw[1];
	if (!imsw[2] && !imsw[3] && !imsw[4]) /* No field components selected */
	    mod_imsw[2] = mod_imsw[3] = mod_imsw[4] = 0;
	else if (   (imsw[2] && !imsw[3] && !imsw[4])     /* Only Br selected */
	         || (!imsw[2] && imsw[3] && !imsw[4])     /* Only Bt selected */
	         || (!imsw[2] && !imsw[3] && imsw[4]) ) { /* Only Bp selected */
	    mod_imsw[2] = 1;
	    mod_imsw[3] = mod_imsw[4] = 0;
	} else
	    mod_imsw[2] = mod_imsw[3] = mod_imsw[4] = 1;
    } else {
	for (ii = 0; ii < 5; ii++)
	    mod_imsw[ii] = imsw[ii];
    }
}


void mk_image(float Cq[], float Cm[], float Br[], float Bt[], float Bp[],
              int ngrid, float lev, float image[], int imsw[])
{
    int j,k;

    j = 0;
    if (imsw[0])
        for (k = 0; k < ngrid; k++) image[j++] = Cq[k];
    if (imsw[0] >= 2) image[j++] = lev; 
    if (imsw[1])
        for (k = 0; k < ngrid; k++) image[j++] = Cm[k];
    if (imsw[2])
        for (k = 0; k < ngrid; k++) image[j++] = Br[k];
    if (imsw[3])
        for (k = 0; k < ngrid; k++) image[j++] = Bt[k];
    if (imsw[4] == 1)
        for (k = 0; k < ngrid; k++) image[j++] = Bp[k];
}


void cp_image(float Cq[], float Cm[], float Br[], float Bt[], float Bp[],
              int ngrid, float *lev, float image[], int imsw[])
{
    int j,k;

    j = 0;
    if (imsw[0])
        for (k = 0; k < ngrid; k++) Cq[k] = image[j++];
    if (imsw[0] >= 2) *lev = image[j++]; 
    if (imsw[1])
        for (k = 0; k < ngrid; k++) Cm[k] = image[j++];
    if (imsw[2])
        for (k = 0; k < ngrid; k++) Br[k] = image[j++];
    if (imsw[3])
        for (k = 0; k < ngrid; k++) Bt[k] = image[j++];
    if (imsw[4])
        for (k = 0; k < ngrid; k++) Bp[k] = image[j++];
}


void mk_data(float I[], float Q[], float U[], float V[],
             spec_info_t spec_info[], int nobs, float data[])
{
    int j,k,n,sp,dp;

    sp = dp = 0;
    for (j = 0; j < nobs; j++) {
	n = spec_info[j].nspec;
	if (spec_info[j].selected) {
            if (spec_info[j].datsel[0])
                for (k = 0; k < n; k++) data[dp++] = I[k+sp];
            if (spec_info[j].datsel[1])
                for (k = 0; k < n; k++) data[dp++] = Q[k+sp];
            if (spec_info[j].datsel[2])
                for (k = 0; k < n; k++) data[dp++] = U[k+sp];
            if (spec_info[j].datsel[3])
                for (k = 0; k < n; k++) data[dp++] = V[k+sp];
        }
	sp += n;
    }
}


void cp_data(float I[], float Q[], float U[], float V[],
             spec_info_t spec_info[], int nobs, float data[])
{
    int j,k,n,sp,dp;

    sp = dp = 0;
    for (j = 0; j < nobs; j++) {
	n = spec_info[j].nspec;
	if (spec_info[j].selected) {
            if (spec_info[j].datsel[0])
                for (k = 0; k < n; k++) I[k+sp] = data[dp++];
            if (spec_info[j].datsel[1])
                for (k = 0; k < n; k++) Q[k+sp] = data[dp++];
            if (spec_info[j].datsel[2])
                for (k = 0; k < n; k++) U[k+sp] = data[dp++];
            if (spec_info[j].datsel[3])
                for (k = 0; k < n; k++) V[k+sp] = data[dp++];
        }
	sp += n;
    }
}


void mk_sigma(spec_info_t spec_info[], int nobs, float sig2[])
{
    int   j,k,dd,dp;
    float sigma2,contin,SN,sigma;
float step, vel; 

    dp = 0;
    for (j = 0; j < nobs; j++)
        if (spec_info[j].selected) {
	    if (spec_info[j].photometric) {
                sigma = spec_info[j].SN;
                sigma2 = sigma*sigma;
	    } else {      /* Spectroscopic data */
                SN = spec_info[j].SN;
                contin = spec_info[j].continuum;
                sigma2 = contin*contin/(SN*SN);
	    }
	    for (dd = 0; dd < 4; dd++)
		if (spec_info[j].datsel[dd]) 
                    for (k = 0; k < spec_info[j].nspec; k++)
                        sig2[dp++] = sigma2;
/*
for (dd = 0; dd < 4; dd++)
if (spec_info[j].datsel[dd]) {
step = (spec_info[j].lamu-spec_info[j].laml)/(spec_info[j].nspec-1); 
for (k = 0; k < spec_info[j].nspec; k++) {
vel = ((spec_info[j].laml+step*k)/spec_info[j].line_centre-1.0)*299792.5; 
if ((vel < -13.0) || (vel > 9.0) || (spec_info[j].photometric)) sig2[dp++] = sigma2;
else                                                            sig2[dp++] = 10.0*sigma2; 
}}
*/
        }
}



void modify_response(float **R, float data[], float odata[],
                     spec_info_t spec_info[], int nobs, float cont[],
                     float dcont[], int imsw[], float Cq[], float Cm[],float lev,
                     int ngrid, int magnetic)
{
    int i,j,k,ii,dd,dp,ip,ipp;
    float c0; 

    fac = mult*lev; upf = 1.0+fac; 

    contin_derivs(spec_info,nobs,cont,dcont,imsw,Cq,Cm,ngrid,magnetic);
/*
printf("%e %e %e %e\n", cont[0],upf,cont[0]*upf,data[0]); 
*/

    dp = ipp = 0;
    for (j = 0; j < nobs; j++)
        if (spec_info[j].selected)
            if (!spec_info[j].photometric) {
                if (imsw[0] == 2)      c0 = cont[j]*fac; 
                else if (imsw[0] == 3) c0 = fac; 
                else                   c0 = 0.0; 
                cont[j] += c0; 
                for (dd = 0; dd < 4; dd++)
	            if (spec_info[j].datsel[dd]) {
                        for (k = 0; k < spec_info[j].nspec; k++) {
                            data[k+dp] = data[k+dp]*spec_info[j].continuum/cont[j];
                            odata[k+dp] = data[k+dp];
	                }
	                if (imsw[0] == 2) for (i = 0; i < ngrid; i++) dcont[i+ipp] *= upf; 
	                for (k = 0; k < spec_info[j].nspec; k++) {
                            ip = 0;
                            for (ii = 0; ii < 5; ii++)
		                if (imsw[ii]) {
                                    if ((ii == 0) && (imsw[0] == 2))
                                        for (i = 0; i < ngrid; i++)  
                                            R[k+dp][i+ip] += dcont[i+ip+ipp]*fac/upf; 
                                    if (ii < 2) 
                                        for (i = 0; i < ngrid; i++)  
                                            R[k+dp][i+ip] = (R[k+dp][i+ip]*spec_info[j].continuum
                                                            - data[k+dp]*dcont[i+ip+ipp])/cont[j];
                                    else
                                        for (i = 0; i < ngrid; i++)
                                            R[k+dp][i+ip] *= spec_info[j].continuum/cont[j];
                                    if ((ii == 0) && (imsw[0] == 2)) 
                                        R[k+dp][ip+ngrid] = (spec_info[j].continuum-data[k+dp]) * 
                                                            mult/upf; 
                                    else if ((ii == 0) && (imsw[0] == 3))
                                        R[k+dp][ip+ngrid] = (spec_info[j].continuum-data[k+dp]) * 
                                                            mult/cont[j]; 
                                    ip += ngrid;
                                }
                        }
	                dp += spec_info[j].nspec;
                    }
                if (imsw[0]) ipp += ngrid;
                if (imsw[1]) ipp += ngrid;
            } else
	        for (dd = 0; dd < 4; dd++)
	            if (spec_info[j].datsel[dd]) dp += spec_info[j].nspec;
}


static void contin_derivs(spec_info_t spec_info[], int nobs, float cont[],
               float dcont[], int imsw[], float Cq[], float Cm[], int ngrid, int magnetic)
{
    int i,j,l,ip;
    float C,phase,area,dCq,dCm,weight;
    cell_info_t *cell;
    static float *dummy1=0,dummy2=0;
    static cell_deriv_t *dummy3=0;

    cell = (cell_info_t *) svector(ngrid,(int) sizeof(cell_info_t));

    ip = 0;
    for (j = 0; j < nobs; j++)
	if (spec_info[j].selected && !spec_info[j].photometric) {

	    cont[j] = 0.0;

	    if (imsw[0]) {
		for (i = 0; i < ngrid; i++) dcont[ip+i] = 0.0;
	        if (imsw[1])
		    for (i = 0; i < ngrid; i++) dcont[ip+ngrid+i] = 0.0;
	    } else if (imsw[1])
	        for (i = 0; i < ngrid; i++) dcont[ip+i] = 0.0;

	    for (l = 1; l <= spec_info[j].nsample; l++) {
	        phase = spec_info[j].phasec + l*(spec_info[j].phasef
			   - spec_info[j].phasec)/(spec_info[j].nsample+1);
	        weight = 1.0 / (float) spec_info[j].nsample;

	        geometry(phase,ngrid,cell,0,dummy1,dummy1,dummy1,
					dummy1,dummy1,dummy1,dummy3);

	        for (i = 0; i < ngrid; i++) {
		    area = cell[i].parea*weight;
        	    if (area > 0.0) {
			if (magnetic)
			    continuum(spec_info[j].line_id,cell[i].mu,
			        Cq[i],Cm[i],&C,&dCq,&dCm,magnetic);
			else
			    continuum(spec_info[j].line_id,cell[i].mu,
			        Cq[i],dummy2,&C,&dCq,&dummy2,magnetic);
			cont[j] += C*area;
	                if (imsw[0]) {
			    dcont[ip+i] += dCq*area;
	                    if (imsw[1]) dcont[ip+ngrid+i] += dCm*area;
                        } else
	                    if (imsw[1]) dcont[ip+i] += dCm*area;
		    }
	        }
	    }
	    if (imsw[0]) ip += ngrid;
	    if (imsw[1]) ip += ngrid;
        }

    free_svector(cell);
}


void unmodify_response(float **R, float odata[],
                       spec_info_t spec_info[], int nobs, float cont[],
                       float dcont[], int imsw[], int ngrid)
{
    int i,j,k,ii,dd,dp,ip,ipp;
    float dc; 

    dp = ipp = 0;
    for (j = 0; j < nobs; j++)
        if (spec_info[j].selected)
            if (!spec_info[j].photometric) {
                for (dd = 0; dd < 4; dd++)
	            if (spec_info[j].datsel[dd]) {
	                for (k = 0; k < spec_info[j].nspec; k++) {
		            ip = 0;
		            for (ii = 0; ii < 5; ii++)
		                if (imsw[ii]) {
		                    if (ii < 2) 
		                        for (i = 0; i < ngrid; i++) 
			                    R[k+dp][i+ip] = (R[k+dp][i+ip]*cont[j]+
                                                            odata[k+dp]*dcont[i+ip+ipp]) /
                                                            spec_info[j].continuum;
		                    else
			                for (i = 0; i < ngrid; i++)
			                    R[k+dp][i+ip] *= cont[j]/spec_info[j].continuum;

                                    if ((ii == 0) && (imsw[0] == 2)) { 
                                        for (i = 0; i < ngrid; i++)  
                                            R[k+dp][i+ip] -= dcont[i+ip+ipp]*fac/upf; 
                                        R[k+dp][ngrid+ip] = cont[j]*mult/upf; 
                                    } else if ((ii == 0) && (imsw[0] == 3))  
                                        R[k+dp][ngrid+ip] = mult; 
                                    ip += ngrid;
                                }
	                }
	                dp += spec_info[j].nspec;
	            }
                if (imsw[0]) ipp += ngrid;
                if (imsw[1]) ipp += ngrid;
            } else
	        for (dd = 0; dd < 4; dd++)
	            if (spec_info[j].datsel[dd]) dp += spec_info[j].nspec;
}


void update_data(float **R, int rows, int cols, float image[], float data[], float off[])
{
    int j,k;
    double sum;

    for (k = 0; k < rows; k++) {
	sum = 0.0;
	for (j = 0; j < cols; j++) 
	    sum += R[k][j]*image[j];
        data[k] = sum+off[k];
    }
}
