#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "structures.h"
#include "file.h"
#include "grid.h"
#include "line.h"
#include "mem_alloc.h"
#include "spec.h"
#include "spec_util.h"
#include "spectrum.h"
#include "utilities.h"

int get_photometry_info(spec_info_t *spec_info, int nobs, float *mag0, int batch); 


int main(argc,argv)
int argc;
char *argv[];
{
    int         ngrid,j,npns,nobs,magnetic,datsw[4],Cm_used,ff_entropy,ff_rec,nl;
    char        filename[80], com1[120];
    int         err,batch;
    float       inc,ve,angle,del_ph,dummy,dbQ,dbM,mag0;
    float       *Cq,*Cm,*Br,*Bt,*Bp,*I,*Q,*U,*V;
    int         nlines,*line_ids,*obs_line,offset;
    char        **line_names,*qnames[2];
    spec_info_t *spec_info;
    spec_sample_t sampling;

    parse_args(argc,argv,&batch,&dummy);

    do {
        err = read_bright(&ngrid,&Cq,&Cm,&Br,&Bt,&Bp,filename,
                                   1,com1,&inc,&ve,&magnetic);
        if (batch) printf("%s\n",filename);
        if (err) printf("File not found.  Try again.\n");
    } while (err);

    image_info(&Cm_used,qnames,&ff_entropy,&ff_rec,batch);
    image_defaults(&dbQ, &dbM, magnetic, Cm_used, qnames, batch);

    init_spectral_lines(batch);

    transform(Cq,Cm,ngrid,magnetic);

    get_spectral_lines(&nlines,&line_names,&line_ids);
    for (j = nl = 0; j < nlines; j++) 
        if (strcmp(line_names[j],"")) nl++; 

    printf("There are %1d spectral lines available.\n",nl);
    obs_line = ivector(0,nlines-1);
    nobs  = 0;
    for (j = 0; j < nlines; j++) {
        if (strcmp(line_names[j],"")) { 
	    printf("Number of phases to use for the line %s : ",line_names[j]);
	    scanf("%d",&(obs_line[j]));
	    if (batch) printf("%d\n",obs_line[j]);
        } else 
            obs_line[j] = 0; 
	nobs += obs_line[j];
    }

    spec_info = (spec_info_t *) svector(nobs,(int) sizeof(spec_info_t));

    offset = 0;
    for (j = 0; j < nlines; j++) {
	if (obs_line[j] > 0) {
	    init_line_data(spec_info,offset,obs_line[j],line_names[j],
				     line_ids[j],magnetic,inc,ve,batch);
	    offset += obs_line[j];
	}
    }

    get_spec_dims(spec_info,nobs,&npns,datsw);


    if (datsw[0]) I = vector(0,npns-1);
    if (datsw[1]) Q = vector(0,npns-1);
    if (datsw[2]) U = vector(0,npns-1);
    if (datsw[3]) V = vector(0,npns-1);

    init_grid(ngrid, inc, ve);

    if ((datsw[1]) || (datsw[2])) {
	printf("What is the polariod angle : ");
	scanf("%f",&angle);
        if (batch) printf("%g\n",angle);
	set_polaroid(angle);
    }

    printf("\nWhat is the largest phase smearing allowed : ");
    scanf("%f",&del_ph);
    if (batch) printf("%g\n",del_ph);
    set_phase_sampling(nobs,spec_info,del_ph);

    printf("What is the oversampling factor : ");
    scanf("%d",&sampling.oversample);
    if (batch) printf("%d\n",sampling.oversample);

    spectra(I,Q,U,V,nobs,spec_info,datsw,sampling,Cq,Cm,Br,Bt,Bp,ngrid,magnetic);
    normalise_spectra(I,Q,U,V,nobs,spec_info,Cq,Cm,ngrid,magnetic);
    if (get_photometry_info(spec_info,nobs,&mag0,batch))
        untransform_photometry(spec_info,nobs,I,mag0,1.0); 

    sprintf(com1,"Synthetic data from image file %s",filename);
    save_spec(I,Q,U,V,nobs,spec_info,filename,1,com1,1);
    if (batch) printf("%s\n",filename);

    end_spectral_lines();

    free_svector((void *) spec_info);
    free_ivector(obs_line,0,nlines-1);
    if (datsw[0]) free_vector(I,0,npns-1);
    if (datsw[1]) free_vector(Q,0,npns-1);
    if (datsw[2]) free_vector(U,0,npns-1);
    if (datsw[3]) free_vector(V,0,npns-1);
    free_vector(Cq,0,ngrid-1);
    if (magnetic) {
        free_vector(Cm,0,ngrid-1);
        free_vector(Br,0,ngrid-1);
        free_vector(Bt,0,ngrid-1);
        free_vector(Bp,0,ngrid-1);
    }
}


static void init_line_data(spec_info,offset,n,name,line_id,
						  magnetic,inc,ve,batch)
spec_info_t spec_info[];
int offset,n,line_id,magnetic,batch;
char name[];
float inc,ve;
{
    int    j,k,photometric,ns,datsw[4],dummy,pre_broadened;
    float  del_ph,nu_max,SN,FWHM;
    double l0,width,lamll,lamuu;
    char   answer[2];

    printf("\nSetting up the information for the line %s.\n",name);

    for (j = 0; j < n; j++) {
	strncpy(spec_info[j+offset].line_name,name,(size_t) MAX_NAME);
	spec_info[j+offset].line_id  = line_id;
	spec_info[j+offset].selected = 1;
    }

    printf("How many spectral data points per phase : ");
    scanf("%d",&ns);
    if (batch) printf("%d\n",ns);

    printf("Do you want the phases to be evenly spaced (y/n) : ");
    scanf("%1s", answer);
    if ((answer[0] == 'Y') || (answer[0] == 'y')) {
	if (batch) printf("yes\n");
	printf("What is the phase change during the observation : ");
	scanf("%f",&del_ph);
	if (batch) printf("%g\n",del_ph);
        for (j = 0; j < n; ++j) {
            spec_info[j+offset].phasec = j / (float) n - del_ph / 2.0;
	    spec_info[j+offset].phasef = j / (float) n + del_ph / 2.0;
	}
    } else {
	if (batch) printf("no\n");
        for (j = 0; j < n; ++j) {
            printf("Enter starting and finishing values for phase %1d : ",j+1);
            scanf("%f %f",&spec_info[j+offset].phasec,
					      &spec_info[j+offset].phasef);
            if (batch) printf("%g %g\n",spec_info[j+offset].phasec,
					      spec_info[j+offset].phasef);
        }
    }

    line_info(line_id,&l0,&width,&pre_broadened,&dummy);

    printf("Do you want to set the wavelength interval automatically (y/n) : ");
    scanf("%1s", answer);
    if ((answer[0] == 'Y') || (answer[0] == 'y')) {
	if (batch) printf("yes\n");
	nu_max = 1.1*l0*ve*sin(inc)/2.998e8 + 4*width;
        lamll = l0 - nu_max;  lamuu = l0 + nu_max;
    } else {
	if (batch) printf("no\n");
        printf("Enter the end wavelength points : ");
        scanf("%lf %lf",&lamll,&lamuu);
	if (batch) printf("%g %g\n",lamll,lamuu);
    }

    printf("Do you want to scale the continuum (y/n) : ");
    scanf("%1s", answer);
    if ((answer[0] == 'Y') || (answer[0] == 'y')) {
	if (batch) printf("yes\n");
        photometric = 0;
    } else {
	if (batch) printf("no\n");
        photometric = 1;
    }
    for (j = 0; j < n; j++) {
	spec_info[j+offset].photometric  = photometric;
    }

    if (photometric) {
        printf("What is the fractional error for these observations : ");
        scanf("%f",&SN);
        if (batch) printf("%g\n",SN);
        FWHM = 0.0; 
    } else {
        printf("What is the S/N value for these observations : ");
        scanf("%f",&SN);
        if (batch) printf("%g\n",SN);
        printf("What is the spectral resolution for these observations : ");
        scanf("%f",&FWHM);
        if (batch) printf("%g\n",FWHM);
        if (FWHM > 0.0) FWHM = l0/FWHM; 
    }


    if (magnetic) {
        datsw[0] = datsw[1] = datsw[2] = datsw[3] = 1;
        get_datsw(datsw,&dummy,batch);
    } else {
        datsw[0] = 1;  datsw[1] = datsw[2] = datsw[3] = 0;
    }

    for (j = 0; j < n; ++j) {
        spec_info[j+offset].nspec  = ns;
        spec_info[j+offset].laml   = lamll;
        spec_info[j+offset].lamu   = lamuu;
        spec_info[j+offset].SN     = SN;
        spec_info[j+offset].FWHM   = FWHM;
	spec_info[j+offset].polariser = 0.0;
	spec_info[j+offset].line_centre = l0;
	spec_info[j+offset].broadened = pre_broadened;
	for (k  = 0; k < 4; k++)
            spec_info[j+offset].datsel[k] = spec_info[j+offset].datsw[k] = datsw[k];
    }
}


static void normalise_spectra(I,Q,U,V,n,spec_info,Cq,Cm,ngrid,magnetic)
float I[],Q[],U[],V[],Cq[],Cm[];
spec_info_t spec_info[];
int n,ngrid,magnetic;
{
    int j,k,nps;
    float *continuum;

    continuum = vector(0,n-1);

    contin(n,spec_info,continuum,ngrid,Cq,Cm,magnetic);

    nps = 0;
    for (j = 0; j < n; j++) {
	if (!spec_info[j].photometric) {
	    if (spec_info[j].datsw[0])
	        for (k = 0; k < spec_info[j].nspec; k++)
		    I[nps+k] /= continuum[j];
	    if (spec_info[j].datsw[1])
	        for (k = 0; k < spec_info[j].nspec; k++)
		    Q[nps+k] /= continuum[j];
	    if (spec_info[j].datsw[2])
	        for (k = 0; k < spec_info[j].nspec; k++)
		    U[nps+k] /= continuum[j];
	    if (spec_info[j].datsw[3])
	        for (k = 0; k < spec_info[j].nspec; k++)
		    V[nps+k] /= continuum[j];
	    spec_info[j].continuum = 1.0;
	} else {
	    spec_info[j].continuum = continuum[j];
	    spec_info[j].SN *= I[nps];
        }
	nps += spec_info[j].nspec;
    }

    free_vector(continuum,0,n-1);
}


int get_photometry_info(spec_info_t *spec_info, int nobs, float *mag0, int batch)
{
    int n, j; 

    for (j = n = 0; j < nobs; j++) 
        if (spec_info[j].photometric && spec_info[j].selected) n++; 

    if (n) {
        printf("Unspotted V magnitude : "); 
        scanf("%f", mag0); 
	if (batch) printf("%f\n",*mag0);
    }  
    return n; 
}

