#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "structures.h"
#include "constants.h"
#include "file.h"
#include "disk.h"
#include "mem_alloc.h"
#include "mem_nl.h"
#include "resp_tom.h"
#include "grid_tom.h"
#include "spec_util.h"
#include "utilities.h"
#include "tomog.h"

char  com2[120] = {"Reconstructed Image"};
char  com3[120] = {"Reconstructed Spectra"};


int main(argc,argv)
int argc;
char *argv[];
{
    float  *Im,*Qm,*Um,*Vm,*Is,*Qs,*Us,*Vs,*x;
    int    j,k,maxit,datsw[4],row,col,n,nobs,nlin_inv,nx,
	   count,linear,n1,n2,batch,existing,npnsm1,err,npns,old_datsw[4],Rsw; 
    char   filename[80],com1[120],answer[2];
    float  inc,vfac,entropy,Chi2,test,Caim,Csq,temp,dbr,L_fac;
    float  **R,*image,*w,*m_data,*s_data,*sig2;
    int    nlines,*line_ids,*line_used,bin;
    char   **line_names;
    spec_info_t   *spec_info;
    float  step, range, **map, vsin; 

      /*  First find out if the job is interactive or batch  */

    parse_args(argc,argv,&batch,&vfac);

      /*  Now read the spectral data to be fitted */

    do {
        err = read_spec(&Im,&Qm,&Um,&Vm,&nobs,&spec_info,filename,1,com1);
        if (batch == 1) printf("%s\n",filename);
        if (err)
            if (batch == 1) {
                printf("Error: File not found.\n");
                exit(1);
            } else
                printf("File not found.  Try again\n");
    } while (err);

/* parameters for inversion */ 
    printf("Stellar vsini (km/s) and inclination (deg) : "); 
    scanf("%f %f", &vsin, &inc); 
    if (batch) printf("%f %f\n", vsin, inc); vsin *= 1000.0; 

    printf("Velocity half-range and step for reconstruction (km/s) : "); 
    scanf("%f %f", &range, &step); 
    if (batch) printf("%f %f\n", range, step); range *= 1000.0; step *= 1000.0; 

    printf("Binning factor for observed spectrum : "); 
    scanf("%d", &bin); 
    if (batch) printf("%d\n", bin); 
    rebin(&Im,&Qm,&Um,&Vm,nobs,spec_info,bin); 

    nx = (float) range/step; nx = 2*nx+1; 
printf("%d\n", nx); 
    init_grid(nx,vsin,inc,step); 
printf("back from init_grid\n"); 
    norm_spec(Im,Qm,Um,Vm,spec_info,nobs,range); 
printf("back from norm_spec\n"); 

    dbr = 0.0001; 
    map = matrix(0, nx-1, 0, nx-1);
    for (k = 0; k < nx; k++) for (j = 0; j < nx; j++) map[k][j] = dbr;

      /*  Now find out some details of the spectral line model and
          the Stokes parameters being fitted.                       */

    init_spectral_lines(batch);
    get_spectral_lines(&nlines,&line_names,&line_ids);
    line_used = ivector(0,nlines-1);
    check_available_lines(spec_info,nobs,line_names,line_ids,nlines,line_used);
    get_lines_to_fit(spec_info,nobs,line_names,line_used,nlines,batch);
    get_stokes_params_to_fit(spec_info,nobs,datsw,batch);

      /*  Initialise the rest of the memory required  */
    get_spec_dims(spec_info,nobs,&npns,old_datsw);
    npnsm1 = npns - 1;

    if (datsw[0])  Is = vector(0,npnsm1);
    if (datsw[1])  Qs = vector(0,npnsm1);
    if (datsw[2])  Us = vector(0,npnsm1);
    if (datsw[3])  Vs = vector(0,npnsm1);

    get_fitted_spec_dims(spec_info,nobs,&row);
    col = nx*nx;
printf("%d %d\n", row, col); 
    R = matrix(0,row-1,0,col-1);
    image  = vector(0,col-1);
    w      = vector(0,col-1);
    m_data = vector(0,row-1);
    s_data = vector(0,row-1);
    sig2   = vector(0,row-1);
    for (k = 0; k < col; k++) w[k] = 1.0; 

    n1 = nx*nx;
    n2 = n1;

    linear = 1; 
    nlin_inv = 0; 

      /*  Finally, read the constants required by mem_nl.  */

    printf("Enter value for L_fac (0 for default) : ");
    scanf("%f", &L_fac);
    if (batch) printf("%g\n",L_fac);
    if (L_fac <= 0.0)
        if (!linear)
            L_fac = 0.1;
        else
            L_fac = 0.3;

    printf("Input values for Caim and Maxit : ");
    scanf("%f %d", &Caim, &maxit);
    if (batch) printf("%g %d\n",Caim,maxit);
    printf("\n");
    ++maxit;

      /*  Initialise variables prior to the main loop.  */

    count = 0;
    test  = 1.0;
    Rsw = 1; 
    for (;;) {    /*  THE MAIN INVERSION ITERATION LOOP  */
        if (count == 0) {
            mk_data(Im,Qm,Um,Vm,spec_info,nobs,m_data);
            mk_sigma(spec_info,nobs,sig2);
        }

        if (count == 0) { 
            response(R,map,nx,Is,Qs,Us,Vs,nobs,spec_info); 
            mk_image(map,nx,image);
            mk_data(Is,Qs,Us,Vs,spec_info,nobs,s_data);
            Csq = 0.0;
            for (j = 0; j < row; j++) {
                temp = (s_data[j] - m_data[j]);
                Csq += temp*temp/sig2[j];
            }
            if (Caim < 0.0) {
                Caim = -Caim*Csq;
                printf("Caim = %8.4e\n\n", Caim/row);
            } else
		Caim *= row;
            printf("           ChiSq         Entropy       Test\n");
            mem_nl(n1,n2,col,row,image,m_data,s_data,sig2,R,w,dbr,dbr,dbr,Caim,-1,
                                              &entropy,&Chi2,&test,nlin_inv,L_fac);
            printf("<%2d> --  %10.4e\n",count,Csq/row);
            count++;
        }

        if ((Csq < Caim*1.01) && (Csq > Caim*0.99) && (test < 0.1)) {
            printf("Found solution\n\n");
            mem_nl(n1,n2,col,row,image,m_data,s_data,sig2,R,w,dbr,dbr,dbr,Caim,-2,
                                              &entropy,&Chi2,&test,nlin_inv,L_fac);
            break;
        }

        if (count >= maxit) {
            if (nlin_inv)
                printf("<%2d> --  %10.4e\n",count,Csq/row);
            else
                --maxit;
            printf("Exceeded %1d iterations\n\n", maxit);
            mem_nl(n1,n2,col,row,image,m_data,s_data,sig2,R,w,dbr,dbr,dbr,Caim,-2,
                                             &entropy,&Chi2,&test,nlin_inv,L_fac);
            break;
        }

        mem_nl(n1,n2,col,row,image,m_data,s_data,sig2,R,w,dbr,dbr,dbr,Caim,Rsw,
                                             &entropy,&Chi2,&test,nlin_inv,L_fac);

	update_data(R,row,col,image,s_data);
        cp_image(map,nx,image);
        cp_data(Is,Qs,Us,Vs,spec_info,nobs,s_data);
        Csq = Chi2;

        printf("<%2d> --  %10.4e   %10.3e   %6.3f\n",count,Csq/row,entropy,test);

        count++;
        Rsw = 0; 
    }

    cp_image(map,nx,image);
    cp_data(Is,Qs,Us,Vs,spec_info,nobs,s_data);

    x = vector(0, nx-1); 
    for (j = 0; j < nx; j++) x[j] = 0.001*step*(j-nx/2); 
    save_map(nx,nx,x,x,map,1,filename,"V\\DX\\U (km/s)", "V\\DY\\U (km/s)", "Doppler map"); 
    if (batch == 1) printf("%s\n",filename);
    free_vector(x, 0, nx-1); 

for (j = n = 0; j < nobs; n += spec_info[j].nspec, j++) 
for (k = 0; k < spec_info[j].nspec; k++) Is[n+k] += 1.0; 

    printf("Do you want to save the synthetic spectra (y/n) : ");
    scanf("%1s", answer);
    if ((answer[0] == 'Y') || (answer[0] == 'y')) {
        if (batch == 1) printf("yes\n");
        save_spec(Is,Qs,Us,Vs,nobs,spec_info,filename,1,com1,1);
        if (batch == 1) printf("%s\n",filename);
    } else
        if (batch == 1) printf("no\n");

    end_spectral_lines();

    free_matrix(R,0,row-1,0,col-1);
    free_vector(image,0,col-1);
    free_vector(w,0,col-1);
    free_vector(m_data,0,row-1);
    free_vector(s_data,0,row-1);
    free_vector(sig2,0,row-1);
    free_svector((void *) spec_info);
    if (datsw[0]) free_vector(Is,0,npnsm1);
    if (datsw[1]) free_vector(Qs,0,npnsm1);
    if (datsw[2]) free_vector(Us,0,npnsm1);
    if (datsw[3]) free_vector(Vs,0,npnsm1);
    free_matrix(map,0,nx-1, 0, nx-1); 
    free_ivector(line_used,0,nlines-1);
}


void check_available_lines(spec_info,nobs,line_names,line_ids,nlines,line_used)
spec_info_t spec_info[];
int nobs,nlines,line_ids[],line_used[];
char *line_names[];
{
    int j,k,found,pre_broadened,linear;
    double lam0,dummy;

    for (k = 0; k < nlines; k++)
	line_used[k] = 0;

    for (j = 0; j < nobs; j++) {
	found = 0;
	for (k = 0; k < nlines; k++)
	    if (!strcmp(spec_info[j].line_name,line_names[k])) {
		found = 1;
	        spec_info[j].line_id = line_ids[k];
		line_used[k] = 1;
		break;
            }
        if (!found) {
	    spec_info[j].selected = 0;
	    printf("There is no line model available for observation %1d (%s).\n",
				                        j+1,spec_info[j].line_name);
        } else {
	    spec_info[j].selected = 1;
            line_info(spec_info[j].line_id,&lam0,&dummy,&pre_broadened,&linear);
	    spec_info[j].broadened = pre_broadened;
	    spec_info[j].linear = linear;
            if (fabs(lam0 - spec_info[j].line_centre) > 0.001*fabs(lam0)) {
                printf("WARNING : The centre wavelengths of the observed data at ");
                printf("observation %1d\n    and spectral model are not the same.\n",
							   j+1);
            }
        }
    }
}


void get_lines_to_fit(spec_info,nobs,line_names,line_used,nlines,batch)
spec_info_t spec_info[];
int nobs,nlines,line_used[],batch;
char *line_names[];
{
    int j,k;
    char answer[2];

    for (k = 0; k < nlines; k++)
	if (line_used[k]) {
	    printf("Do you want to use the %s observations (y/n) : ",line_names[k]);
	    scanf("%1s",answer);
            if ((answer[0] == 'Y') || (answer[0] == 'y')) {
		if (batch) printf("yes\n");
            } else {
                if (batch) printf("no\n");
		for (j = 0; j < nobs; j++)
	            if (!strcmp(spec_info[j].line_name,line_names[k]))
	                spec_info[j].selected = 0;
            }
	}
}


void get_stokes_params_to_fit(spec_info,nobs,datsw,batch)
spec_info_t spec_info[];
int nobs,datsw[4],batch;
{
    int j,k,selected,dummy;

    get_selected_stokes(spec_info,nobs,datsw);
    get_datsw(datsw,&dummy,batch);
    for (j = 0; j < nobs; j++)
	if (spec_info[j].selected) {
	    selected = 0;
	    for (k = 0; k < 4; k++)
		if (spec_info[j].datsw[k])
		    if (datsw[k]) {
		        selected = 1;
			spec_info[j].datsel[k] = 1;
                    } else
			spec_info[j].datsel[k] = 0;
            if (!selected)
		spec_info[j].selected = 0;
        }
}


static void set_pixel_weights(w,weight,ngrid,n_image)
float w[];
int weight,ngrid,n_image;
{
    int i,j;
    float *area;

    if (weight) {
	area = vector(0,ngrid-1);
	areas(ngrid,area);
	for (j = 0; j < n_image; j++)
	    for (i = 0; i < ngrid; i++) {
		w[i+j*ngrid] = area[i];
            }
	free_vector(area,0,ngrid-1);
    } else {
	for (i = 0; i < ngrid*n_image; i++)
	    w[i] = 1.0;
    }
}


int get_photometry_info(spec_info_t *spec_info, int nobs, float *mag0, float *wphot, 
                        int batch)
{
    int n, j; 

    for (j = n = 0; j < nobs; j++) 
        if (spec_info[j].photometric && spec_info[j].selected) n++; 

    if (n) {
        printf("Unspotted V magnitude : "); 
        scanf("%f", mag0); 
        if (batch) printf("%.3f\n", *mag0); 

        printf("Weight on photometry : "); 
        scanf("%f", wphot); 
        if (batch) printf("%.3f\n", *wphot); 
    } else 
        *wphot = 1.0;  
    return n; 
}


static void norm_spec(float *I, float *Q, float *U, float *V, 
                      spec_info_t *info, int nobs, float range) 
{
    int   j, n, p; 
    float x, tmp, sx, sy, sx2, sxy, y, step, mult, a, b, d, w; 

    mult = 10000.0; 
    tmp = sx = sy = sx2 = sxy = 0.0;
    for (j = n = 0; j < nobs; n += info[j].nspec, j++) {
        if (!info[j].datsw[0]) { 
            printf("Cannot normalise spectrum: Stokes I not supplied\n"); 
            exit(1); 
        } 
        step = (info[j].lamu-info[j].laml) / (info[j].nspec-1);
        for (p = 0; p < info[j].nspec; p++) {
            x = C_l*((info[j].laml+step*p)/info[j].line_centre-1.0)/mult;
            y = I[n+p]; 
            if (fabs(x) > range/mult) { 
                w = 1.0;
                sx  += w*x;
                sy  += w*y;
                sx2 += w*x*x;
                sxy += w*x*y;
                tmp += w;
            } 
        }
        d = tmp*sx2-sx*sx;
        a = (tmp*sxy-sx*sy)/d; b = (sx2*sy-sx*sxy)/d;

        for (p = 0; p < info[j].nspec; p++) {
            x = C_l*((info[j].laml+step*p)/info[j].line_centre-1.0)/mult;
            I[n+p] /= a*x+b; I[n+p] -= 1.0; 
            if (info[j].datsw[1]) Q[n+p] /= a*x+b; 
            if (info[j].datsw[2]) U[n+p] /= a*x+b; 
            if (info[j].datsw[3]) V[n+p] /= a*x+b; 
        } 
printf("a, b = %f %f\n", a,b); 
    }
}


void rebin(float **I, float **Q, float **U, float **V, int nobs, spec_info_t *info, int bin)
{
    int   j, k, n, p, nold, datsw[4]; 
    float *Inew, *Qnew, *Unew, *Vnew, step; 

    if (bin <= 1) return; 
    for (k = 0; k < 4; k++) datsw[k] = 0; 
    for (j = n = 0; j < nobs; j++) {
        if (info[j].nspec < bin) n += 1; 
        else                     n += info[j].nspec/bin; 
        for (k = 0; k < 4; k++) if (info[j].datsw[k]) datsw[k] = 1;
    } 

    if (datsw[0]) { Inew = vector(0,n-1); for (j = 0; j < n; j++) Inew[j] = 0.0; } 
    if (datsw[1]) { Qnew = vector(0,n-1); for (j = 0; j < n; j++) Qnew[j] = 0.0; } 
    if (datsw[2]) { Unew = vector(0,n-1); for (j = 0; j < n; j++) Unew[j] = 0.0; } 
    if (datsw[3]) { Vnew = vector(0,n-1); for (j = 0; j < n; j++) Vnew[j] = 0.0; } 

    for (j = n = nold = 0; j < nobs; j++) 
        if (info[j].nspec < bin) { 
            if (datsw[0]) for (k = 0; k < info[j].nspec; k++) Inew[n] += (*I)[nold+k]/info[j].nspec;
            if (datsw[1]) for (k = 0; k < info[j].nspec; k++) Qnew[n] += (*Q)[nold+k]/info[j].nspec;
            if (datsw[2]) for (k = 0; k < info[j].nspec; k++) Unew[n] += (*U)[nold+k]/info[j].nspec;
            if (datsw[3]) for (k = 0; k < info[j].nspec; k++) Vnew[n] += (*V)[nold+k]/info[j].nspec;
            info[j].laml = info[j].lamu = 0.5*(info[j].laml+info[j].lamu); 
/*
            info[j].SN *= sqrt((float) info[j].nspec); 
*/
            n += 1; 
            nold += info[j].nspec; 
            info[j].nspec = 1; 
        } else { 
            if (datsw[0]) for (k = 0; k+bin-1 < info[j].nspec; k += bin) 
                    for (p = 0; p < bin; p++) Inew[n+k/bin] += (*I)[nold+k+p]/bin; 
            if (datsw[1]) for (k = 0; k+bin-1 < info[j].nspec; k += bin) 
                    for (p = 0; p < bin; p++) Qnew[n+k/bin] += (*Q)[nold+k+p]/bin; 
            if (datsw[2]) for (k = 0; k+bin-1 < info[j].nspec; k += bin) 
                    for (p = 0; p < bin; p++) Unew[n+k/bin] += (*U)[nold+k+p]/bin; 
            if (datsw[3]) for (k = 0; k+bin-1 < info[j].nspec; k += bin) 
                    for (p = 0; p < bin; p++) Vnew[n+k/bin] += (*V)[nold+k+p]/bin; 
            step = (info[j].lamu-info[j].laml)/(info[j].nspec-1); 
            info[j].lamu = info[j].laml + step*(k-bin) + 0.25*step*bin*(bin-1); 
            info[j].laml += 0.25*step*bin*(bin-1); 
/*
            info[j].SN *= sqrt((float) bin); 
*/
            n += info[j].nspec/bin; 
            nold += info[j].nspec; 
            info[j].nspec /= bin; 
        } 
    if (datsw[0]) { free_vector(*I, 0, nold-1); *I = Inew; } 
    if (datsw[1]) { free_vector(*Q, 0, nold-1); *Q = Qnew; } 
    if (datsw[2]) { free_vector(*U, 0, nold-1); *U = Unew; } 
    if (datsw[3]) { free_vector(*V, 0, nold-1); *V = Vnew; } 
}
