/*

This file contains utility routines used by more than one
of the three major programs, syn, spec and zdi.
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "constants.h"
#include "structures.h"
#include "file.h"
#include "mem_alloc.h"
#include "utilities.h"


void parse_args(argc,argv,batch,vfac)
int argc,*batch;
char *argv[];
float *vfac;
{
    int i,mps;

#ifdef V_IN_MPS  /* Switch to set velocity units to m/s */
    mps = 1;
#else
    mps = 0;   /* Default is km/s */
#endif

    *batch = 0;
    for (i = 1; i < argc; i++)
	if ((argv[i][0] == '-') && (argv[i][1] == 'b'))
	    *batch = 1;
        else if ((argv[i][0] == '-') && (argv[i][1] == 'm'))
	    mps = 1;
        else if ((argv[i][0] == '-') && (argv[i][1] == 'k'))
	    mps = 0;
        else
	    fprintf(stderr,"Warning: unknown command line option %s\n",argv[i]);

    if (mps)
        *vfac = 1.0;
    else
	*vfac = 1000.0;  /* Need to multiply by 1000 to convert km/s to m/s */

#ifdef ASK_IF_BATCH      /* An alternative for systems without command line options */
    if (!*batch) {
        printf("Are you running interactively (y/n) : ");
	scanf("%1s", answer);
	if ((answer[0] == 'Y') || (answer[0] == 'y'))
	    *batch = 0;
	else {
	    *batch = 1;
	    printf("no\n");
	}
    }
#endif
}


void get_imsw(imsw,n_image,Cm_used,batch)
int imsw[],*n_image,Cm_used,batch;
{
    char answer[20];
    int c,i,j;
    void exit();

    *n_image = 0;
    for (i = 0; i < 5; i++)
        imsw[i] = 0;

    if (Cm_used)
        printf("Select image parameters to be fitted (any combination of QMRTP) : ");
    else
        printf("Select image parameters to be fitted (any combination of QRTP) : ");

    scanf("%20s", answer);
    j = 0;
    while (answer[j] != '\0') {
        c = answer[j];
        if ((c == 'Q') || (c == 'q')) {
            if (imsw[0] == 0) {
                ++(*n_image);
                if (batch == 1) printf("Q");
            }
            imsw[0] = 1;
        } else if (Cm_used && ((c == 'M') || (c == 'm'))) {
            if (imsw[1] == 0) {
                ++(*n_image);
                if (batch == 1) printf("M");
            }
            imsw[1] = 1;
        } else if ((c == 'R') || (c == 'r')) {
            if (imsw[2] == 0) {
                ++(*n_image);
                if (batch == 1) printf("R");
            }
            imsw[2] = 1;
        } else if ((c == 'T') || (c == 't')) {
            if (imsw[3] == 0) {
                ++(*n_image);
                if (batch == 1) printf("T");
            }
            imsw[3] = 1;
        } else if ((c == 'P') || (c == 'p')) {
            if (imsw[4] == 0) {
                ++(*n_image);
                if (batch == 1) printf("P");
            }
            imsw[4] = 1;
        } else if ((c != ' ') && (c != ','))
            fprintf(stderr,"Unknown parameter <%c> ignored\n", c);

        j++;
    }
    if (batch == 1) printf("\n");
    if (*n_image == 0)
        if (batch == 0) {
            fprintf(stderr,"You must choose at least one parameter.  Try again\n");
            get_imsw(imsw,n_image,Cm_used,batch);
        } else {
            fprintf(stderr,"You must choose at least one parameter.\n");
            exit(1);
        }
}


void get_datsw(datsw,n_data,batch)
int datsw[],*n_data,batch;
{
    char answer[20];
    int  c,i,j,new_datsw[4];
    void exit();

    *n_data = 0;
    for (i = 0; i < 4; i++) {
        new_datsw[i] = 0;
        if (datsw[i] == 1)
            ++(*n_data);
    }
    if (*n_data == 0) {
        fprintf(stderr,"Error: No Stokes parameters available to fit\n");
        exit(1);
    }
    if (*n_data == 1)
        return;

    *n_data = 0;
    printf("Select Stokes parameters to use (any combination of ");
    if (datsw[0] == 1)  printf("I");
    if (datsw[1] == 1)  printf("Q");
    if (datsw[2] == 1)  printf("U");
    if (datsw[3] == 1)  printf("V");
    printf(") : ");
    scanf("%20s", answer);
    j = 0;
    while (answer[j] != '\0') {
        c = answer[j];
        if ((datsw[0] == 1) && ((c == 'I') || (c == 'i'))) {
            if (new_datsw[0] == 0) {
                ++(*n_data);
                if (batch == 1) printf("I");
            }
            new_datsw[0] = 1;
        } else if ((datsw[1] == 1) && ((c == 'Q') || (c == 'q'))) {
            if (new_datsw[1] == 0) {
                ++(*n_data);
                if (batch == 1) printf("Q");
            }
            new_datsw[1] = 1;
        } else if ((datsw[2] == 1) && ((c == 'U') || (c == 'u'))) {
            if (new_datsw[2] == 0) {
                ++(*n_data);
                if (batch == 1) printf("U");
            }
            new_datsw[2] = 1;
        } else if ((datsw[3] == 1) && ((c == 'V') || (c == 'v'))) {
            if (new_datsw[3] == 0) {
                ++(*n_data);
                if (batch == 1) printf("V");
            }
            new_datsw[3] = 1;
        } else if ((c != ' ') && (c != ','))
            fprintf(stderr,"Unknown parameter <%c> ignored\n", c);

        j++;
    }
    if (batch == 1) printf("\n");
    if (*n_data == 0)
        if (batch == 0) {
            fprintf(stderr,"You must choose at least one parameter.  Try again\n");
            get_datsw(datsw,n_data,batch);
        } else {
            fprintf(stderr,"You must choose at least one parameter.\n");
            exit(1);
        }
    for (j = 0; j < 4; j++)
        datsw[j] = new_datsw[j];
}


void init_image(Cq,Cm,Br,Bt,Bp,ngrid,inc,ve,magnetic,existing,batch,vfac)
float **Cq,**Cm,**Br,**Bt,**Bp;  /* Pointers to arrays to be initialised */
int *ngrid;          /* Number of grid points */
float *inc,*ve;      /* Inclination and equatorial rotation velocity */
int *magnetic;       /* Switch to indicate a magnetic star */
int *existing;       /* Switch to indicate that image was read from a file */
int batch;           /* Echo input if batch is true */
float vfac;          /* Multiplication factor to convert velocity to m/s */
{
    char filename[80], com[120], answer[2];
    int  j,mag_file,err;

    printf("Do you want to make a magnetic image (y/n) : ");
    scanf("%1s", answer);
    if ((answer[0] == 'Y') || (answer[0] == 'y')) {
        if (batch) printf("yes\n");
        *magnetic = 1;
    } else {
	if (batch) printf("no\n");
	*magnetic = 0;
    }

    printf("Do you want to read an old image file (y/n) : ");
    scanf("%1s", answer);
    if ((answer[0] == 'Y') || (answer[0] == 'y')) {
        if (batch) printf("yes\n");
	*existing = 1;

        do {
            err = read_bright(ngrid,Cq,Cm,Br,Bt,Bp,filename,1,
						com,inc,ve,&mag_file);
	    if (batch) printf("%s\n",filename);
            if (err)
	        if (batch) {
		    printf("Error: File not found.\n");
		    exit(1);
		} else
		    printf("File not found.  Try again\n");
        } while (err);
	if (*magnetic && !mag_file) { /* Adding magnetic features to non-magnetic image */
	    *Cm = vector(0,*ngrid-1);  /* Allocate storage for the magnetic components */
	    *Br = vector(0,*ngrid-1);
	    *Bt = vector(0,*ngrid-1);
            *Bp = vector(0,*ngrid-1);
	    for (j = 0; j < *ngrid; ++j)  /* Initialise the magnetic components */
                (*Cm)[j] = (*Br)[j] = (*Bt)[j] = (*Bp)[j] = 0.0;
        }

    } else {
        if (batch) printf("no\n");
	*existing = 0;

        printf("Input ngrid, incl and vsin(i) : ");
        scanf("%d %f %f", ngrid, inc, ve);
        if (*ngrid <= 0) *ngrid = 10;
        if (batch) printf("%d %g %g\n",*ngrid,*inc,*ve);
        *inc = TO_RAD(*inc);
	if (*ve > 0.0)
	    if (*inc != 0.0)
	        *ve = (*ve)*vfac/sin(*inc);
            else
		*ve = 0.0;  /* For inclination zero, ve is arbitrary */
	else
	    *ve = -(*ve)*vfac;  /* Negative values signal a straight velocity */

        *Cq = vector(0,*ngrid-1);
        if (*magnetic) {
	    *Cm = vector(0,*ngrid-1);
	    *Br = vector(0,*ngrid-1);
	    *Bt = vector(0,*ngrid-1);
	    *Bp = vector(0,*ngrid-1);
        }
    }
}
