#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "structures.h"
#include "constants.h"
#include "file.h"
#include "grid.h"
#include "line.h"
#include "mem_alloc.h"
#include "mem_nl2.h"
#include "response.h"
#include "spec_util.h"
#include "utilities.h"
#include "zdi.h"

char  com2[120] = {"Reconstructed Image"};
char  com3[120] = {"Reconstructed Spectra"};

int get_photometry_info(spec_info_t *spec_info, int nobs, float *mag0, float *wphot, 
                        int batch); 

int main(argc,argv)
int argc;
char *argv[];
{
    float  *Cq,*Cm,*Br,*Bt,*Bp,*Im,*Qm,*Um,*Vm,*Is,*Qs,*Us,*Vs;
    int    ngrid,j,k,nobs,magnetic,maxit,imsw[5],datsw[4],row,col,n,
	   count,linear,nlin_inv,Rsw,n_image,n1,n2,batch,existing,npnsm1,
	   err,fault,index,ind,npns,old_datsw[4],ndc,spectroscopic,nphot,
	   quasi_linear,ql_itts,weight,Cm_used,ff_entropy,ff_rec,calc_all,
           calc_quasi;
    char   filename[80],com1[120],answer[2],*qnames[2];
    float  inc,ve,vfac,entropy,Chi2,test,mag0,wphot,
           dbr,Caim,Csq,temp,B,*C,dbQ,dbQ_phot,dbQ_spot,dbM,dbR,dbT,dbP,
           del_C,del_B,pch,Csq_old,L_fac,change,del_ph,Bscale;
    float  **R,*image,*w,*m_data,*s_data,*sig2,*old_image,*old_data,
	   *cont,*dcont,spot,alpha,tmp;
    int    nlines,*line_ids,*line_used;
    char   **line_names;
    spec_info_t   *spec_info;
    spec_sample_t sampling;

      /*  First find out if the job is interactive or batch  */

    parse_args(argc,argv,&batch,&vfac);

      /* Now we get all the details of the image to be reconstructed */

    init_image(&Cq,&Cm,&Br,&Bt,&Bp,&ngrid,&inc,&ve,&magnetic,&existing,batch,vfac);

    image_info(&Cm_used,qnames,&ff_entropy,&ff_rec,batch);

    if (magnetic) {
        get_imsw(imsw,&n_image,Cm_used,batch);
    } else {
        for (j = 0; j < 5; j++)
            imsw[j] = 0;
        imsw[0] = 1;
        n_image = 1;
    }

    /* Finds the default */ 

    image_defaults(&dbQ, &dbM, magnetic, Cm_used, qnames, batch); 

    if (imsw[2] || imsw[3] || imsw[4]) {
	 printf("What is a typical value for the magnetic field strength : ");
	 scanf("%f",&Bscale);
	 if (batch) printf("%g\n",Bscale);
    } else if (ff_entropy == -1) {
         Bscale = 5.0; 
    } else 
         Bscale = 0.0; 
    printf("\n");
    if (imsw[2]) dbR = 0.01*Bscale;  else  dbR = 0.0;
    if (imsw[3] && !imsw[2]) dbT = 0.01*Bscale;  else  dbT = 0.0;
    if (imsw[4] && !imsw[2]) dbP = 0.01*Bscale;  else  dbP = 0.0;
printf("C=%f\n", C); 

    if (!existing)
        for (j = 0; j < ngrid; ++j) {
            if (ff_entropy > 0) { 
                Cq[j] = 0.999*dbQ;
                if (ff_rec == 1) if (Cq[j] >= 1.0) Cq[j] = 0.999; 
            } else 
                Cq[j] = dbQ;
            if (magnetic) {
                Cm[j] = dbM;
                Br[j] = dbR;
                Bt[j] = dbT;
                Bp[j] = dbP;
            }
        }
    else { 
        if (ff_entropy > 0)
            for (j = 0; j < ngrid; ++j) 
		if (Cq[j] >= 0.999*dbQ)       Cq[j] = 0.999*dbQ;
		else if (Cq[j] <= 0.001*dbQ)  Cq[j] = 0.001*dbQ;

                  /* Modify zero values for the magnetic field in an existing image */
	if (magnetic)  /* Values will only be changed if they're being fitted */
            for (j = 0; j < ngrid; ++j) {
		if (Br[j] == 0.0)  Br[j] = dbR;
		if (Bt[j] == 0.0)  Bt[j] = dbT;
		if (Bp[j] == 0.0)  Bp[j] = dbP;
	    }
    }

    transform(Cq,Cm,ngrid,magnetic);

    transform(&dbQ,&dbM,1,magnetic);
    if (imsw[1] && !imsw[0])
        dbr = dbM;
    else 
        dbr = dbQ;
  
    C = vector(0, ngrid-1); 
    for (j = 0; j < ngrid; j++) if (Cq[j] >= 0.999*dbQ)      C[j] = 0.999*dbQ; 
                                else if (Cq[j] <= 0.001*dbQ) C[j] = 0.001*dbQ; 
                                else                         C[j] = Cq[j]; 

      /*  Initialise the grid routines  */
    init_grid(ngrid,inc,ve);
    init_diffrot(batch); 

      /*  Now read the spectral data to be fitted */

    do {
        err = read_spec(&Im,&Qm,&Um,&Vm,&nobs,&spec_info,filename,1,com1);
        if (batch == 1) printf("%s\n",filename);
        if (err)
            if (batch == 1) {
                printf("Error: File not found.\n");
                exit(1);
            } else
                printf("File not found.  Try again\n");
    } while (err);

      /*  Now find out some details of the spectral line model and
          the Stokes parameters being fitted.                       */

    init_spectral_lines(batch);
    get_spectral_lines(&nlines,&line_names,&line_ids);
    line_used = ivector(0,nlines-1);
    check_available_lines(spec_info,nobs,line_names,line_ids,nlines,line_used);
    get_lines_to_fit(spec_info,nobs,line_names,line_used,nlines,batch);
    get_stokes_params_to_fit(spec_info,nobs,datsw,batch);

    if (nphot = get_photometry_info(spec_info,nobs,&mag0,&wphot,batch)) 
        transform_photometry(spec_info,nobs,Im,mag0,wphot); 

    printf("What is the largest phase smearing allowed : ");
    scanf("%f",&del_ph);
    if (batch) printf("%f\n",del_ph);
    set_phase_sampling(nobs,spec_info,del_ph);

    printf("What is the oversampling factor : ");
    scanf("%d",&sampling.oversample);
    if (batch) printf("%1d\n",sampling.oversample);

    printf("\n");

      /*  Initialise the rest of the memory required  */
    get_spec_dims(spec_info,nobs,&npns,old_datsw);
    npnsm1 = npns - 1;

    if (datsw[0])  Is = vector(0,npnsm1);
    if (datsw[1])  Qs = vector(0,npnsm1);
    if (datsw[2])  Us = vector(0,npnsm1);
    if (datsw[3])  Vs = vector(0,npnsm1);

    get_fitted_spec_dims(spec_info,nobs,&row);
    col = n_image*ngrid;
    R = matrix(0,row-1,0,col-1);
    image  = vector(0,col-1);
    w      = vector(0,col-1);
    m_data = vector(0,row-1);
    s_data = vector(0,row-1);
    sig2   = vector(0,row-1);

    check_spectroscopic(spec_info,nobs,imsw,ngrid,&spectroscopic,&ndc);
    if (spectroscopic) {
        old_data = vector(0,row-1);
	cont  = vector(0,nobs-1);
	if (ndc) dcont = vector(0,ndc-1);
    }

      /*  Find n1, the number of image pixels using p log p entropy.  */

    n1 = 0;
    if (imsw[0])
        n1 += ngrid;
    if (imsw[1])
        n1 += ngrid;

    if (ff_entropy > 0) {
        n2 = n1;
	n1 = 0;
    } else if (ff_entropy < 0) 
        n1 = n2 = 0;
    else 
        n2 = n1;

      /*  Find the maximum permitted jumps in abundance and magnetic
          field per iteration if the model is non-linear.             */

    check_linear(spec_info,nobs,&linear,&quasi_linear);
    if (quasi_linear) {
	printf("Input the number of iterations per response matrix calculation : ");
	scanf("%d",&ql_itts);
	if (batch) printf("%d\n",ql_itts);
    }

    if (linear && !spectroscopic)  /* Check if the inversion is linear */
	nlin_inv = 0;
    else
	nlin_inv = 1;

    if (!linear) {
        if (imsw[0] || imsw[1]) {
            printf("Enter the maximum change in each %s pixel : ",qnames[0]);
            scanf("%f",&del_C);
            if (batch) printf("%g\n",del_C);
        }
        if (imsw[2] || imsw[3] || imsw[4]) {
            printf("Enter the maximum change in each magnetic pixel : ");
            scanf("%f",&del_B);
            if (batch) printf("%g\n",del_B);
        }
        old_image = vector(0,col-1);
        pch = 1.0;
    }

      /*  Set up the pixel weights for mem_nl. */

    printf("Do you want to weight the pixels by the cell area (y/n) : ");
    scanf("%1s", answer);
    if ((answer[0] == 'Y') || (answer[0] == 'y')) {
        if (batch)  printf("yes\n");
        weight = 1;
    } else {
        if (batch) printf("no\n");
        weight = 0;
    }
    set_pixel_weights(w,weight,ngrid,n_image);

      /*  Finally, read the constants required by mem_nl.  */

    printf("Enter value for L_fac (0 for default) : ");
    scanf("%f", &L_fac);
    if (batch) printf("%g\n",L_fac);
    if (L_fac <= 0.0)
        if (!linear)
            L_fac = 0.1;
        else
            L_fac = 0.3;

    B = 0.1*Bscale;  /* Constant used in max entropy routine */
    /* B =     Bscale; */ /* Constant used in max entropy routine */

    printf("Input values for Caim and Maxit : ");
    scanf("%f %d", &Caim, &maxit);
    if (batch) printf("%g %d\n",Caim,maxit);
    printf("\n");

    if (!nlin_inv)
        ++maxit;

      /*  Initialise variables prior to the main loop.  */

    count = 0;
    test  = 1.0;
    Rsw   = 1;
    fault = 0;
    calc_all = 1;

    for (;;) {    /*  THE MAIN INVERSION ITERATION LOOP  */
        if (count == 0) {
            mk_data(Im,Qm,Um,Vm,spec_info,nobs,m_data);
            mk_sigma(spec_info,nobs,sig2);
        }

	if (quasi_linear && (count % ql_itts) == 0) {
	    Rsw = 1;  /* Signals to calculate a new response matrix */
	    calc_quasi = 1;
        } else
	    calc_quasi = 0;

        if (!linear || Rsw) {
            response(R,Cq,Cm,Br,Bt,Bp,Is,Qs,Us,Vs,ngrid,nobs,spec_info,
	                      sampling,imsw,magnetic,calc_all,calc_quasi);
            mk_image(Cq,Cm,Br,Bt,Bp,ngrid,image,imsw);
            mk_data(Is,Qs,Us,Vs,spec_info,nobs,s_data);
        }

	if (spectroscopic)
	    modify_response(R,s_data,old_data,spec_info,nobs,cont,dcont,
	         			       imsw,Cq,Cm,ngrid,magnetic);

        if ((count == 0) || nlin_inv) {
            Csq = 0.0;
            for (j = 0; j < row; j++) {
                temp = (s_data[j] - m_data[j]);
                Csq += temp*temp/sig2[j];
            }
        }

/*
        if (!linear && (count > 0) && (Csq > Caim) && (Csq > Csq_old)) {
            if (pch > 0.01) {
                pch /= 2.0;
                index = 0;
                for (k = 0; k < 5; k++) {
                    if (!imsw[k])
                        continue;
                    if (k < 2)
                        change = pch*del_C;
                    else
                        change = pch*del_B;
                    for (j = 0; j < ngrid; j++) {
                        if (image[j+index] > (old_image[j+index] + change))
                            image[j+index] = old_image[j+index] + change;
                        if (image[j+index] < (old_image[j+index] - change))
                            image[j+index] = old_image[j+index] - change;
                    }
                    index += ngrid;
                }
                fault++;
                cp_image(Cq,Cm,Br,Bt,Bp,ngrid,image,imsw);
                continue;   *//* Return to the start of the for loop *//*
            }
        }
*/

        if (count == 0) {
            if (Caim < 0.0) {
                Caim = -Caim*Csq;
                printf("Caim = %8.4e\n\n", Caim/(row+nphot*(wphot-1.0)));
            } else
		Caim *= row+nphot*(wphot-1.0);
            if (ff_entropy) 
                  printf("           ChiSq         Entropy     Spots     Test\n");
            else  
                printf("           ChiSq         Entropy     <Bmod>    Test\n");
        }
printf("here\n"); 

        if (!nlin_inv && (count == 0)) {  /* Initialise MEM storage */
            mem_nl(n1,n2,col,row,image,m_data,s_data,sig2,R,w,dbr,B,C,Caim,-1,
                                              &entropy,&Chi2,&test,nlin_inv,L_fac);
            printf("<%2d> --  %10.4e\n",count,Csq/(row+nphot*(wphot-1.0)));
            count++;
        }

        if ((Csq < Caim*1.001) && (Csq > Caim*0.999) && (test < 0.01)) {
            if (nlin_inv)
                printf("<%2d> --  %10.4e\n",count,Csq/(row+nphot*(wphot-1.0)));
            printf("Found solution\n\n");
            if (!nlin_inv) /* De-allocate MEM storage */
                mem_nl(n1,n2,col,row,image,m_data,s_data,sig2,R,w,dbr,B,C,Caim,-2,
                                              &entropy,&Chi2,&test,nlin_inv,L_fac);
            break;
        }

        if (count >= maxit) {
            if (nlin_inv)
                printf("<%2d> --  %10.4e\n",count,Csq/(row+nphot*(wphot-1.0)));
            else
                --maxit;
            printf("Exceeded %1d iterations\n\n", maxit);
            if (!nlin_inv) /* De-allocate MEM storage */
                mem_nl(n1,n2,col,row,image,m_data,s_data,sig2,R,w,dbr,B,C,Caim,-2,
                                             &entropy,&Chi2,&test,nlin_inv,L_fac);
            break;
        }

        if (!linear) {
            for (j = 0; j < col; j++)
                old_image[j] = image[j];
            Csq_old = Csq;
        }
printf("going to mem_nl\n"); 
        mem_nl(n1,n2,col,row,image,m_data,s_data,sig2,R,w,dbr,B,C,Caim,Rsw,
                                             &entropy,&Chi2,&test,nlin_inv,L_fac);

	if (spectroscopic) {
	    unmodify_response(R,old_data,spec_info,nobs,cont,dcont,imsw,ngrid);
	    update_data(R,row,col,image,s_data);
            cp_image(Cq,Cm,Br,Bt,Bp,ngrid,image,imsw);
            cp_data(Is,Qs,Us,Vs,spec_info,nobs,s_data);
        }

        if (!nlin_inv) {
            Csq = Chi2;
            cp_image(Cq,Cm,Br,Bt,Bp,ngrid,image,imsw);
            cp_data(Is,Qs,Us,Vs,spec_info,nobs,s_data);
        }

        if (ff_entropy) { 
            spot = 0.0; 
            for (j = 0; j < ngrid; j++) spot += (double) (1.0-Cq[j])/ngrid; 
            printf("<%2d> --  %10.4e   %10.3e   %7.4f   %6.3f",count,
                   Csq/(row+nphot*(wphot-1.0)),entropy,spot,test);
        } else {
            spot = 0.0; 
            for (j = 0; j < ngrid; j++) { 
                tmp = 0.0; 
                if (imsw[2]) tmp += Br[j]*Br[j]; 
                if (imsw[3]) tmp += Bp[j]*Bp[j]; 
                if (imsw[4]) tmp += Bt[j]*Bt[j]; 
                spot += sqrt(tmp)/ngrid; 
            } 
            printf("<%2d> --  %10.4e   %10.3e   %7.2f   %6.3f",count,
                   Csq/(row+nphot*(wphot-1.0)),entropy,spot,test);
        } 

        if (!linear && (fault > 0)) {
            printf("      ");
            for (j = 0; j < fault; j++)
                printf("-");
            for (j = 2; j < fault; j++) {
                printf("+");
                pch *= 2.0;
            }
        }
        printf("\n");
        fault = 0;

        if (!linear) {
            index = 0;
            for (k = 0; k < 5; k++) {
                if (!imsw[k]) continue;
                if (k < 2) change = pch*del_C;
                else       change = pch*del_B;
                for (j = 0; j < ngrid; j++) {
                    if (image[j+index] > (old_image[j+index] + change))
                        image[j+index] = old_image[j+index] + change;
                    if (image[j+index] < (old_image[j+index] - change))
                        image[j+index] = old_image[j+index] - change;
                }
                index += ngrid;
            }
/*
printf("Shouldn't be here\n"); 
                change = pch*del_C;
                for (j = 0; j < ngrid; j++) { 
                    tmp = fabs(image[j]-old_image[j]); 
                    if (alpha < tmp) alpha = tmp; 
                } 
                alpha = change/alpha; 
                if (alpha < 1.0)  
                    for (j = 0; j < ngrid; j++) 
                        image[j] = old_image[j]*(1.0-alpha)+image[j]*alpha; 
                index += ngrid; 
            } 
            if (imsw[1]) index += ngrid; 
            if (imsw[2] || imsw[3] || imsw[4]) { 
printf("Should be here\n"); 
                alpha = 0.0; ind = 0; 
                change = pch*del_B;
                for (k = 2; k < 5; k++) 
                    if (imsw[k]) { 
                        for (j = 0; j < ngrid; j++) { 
                            tmp = fabs(image[j+index+ind]-old_image[j+index+ind]); 
                            if (alpha < tmp) alpha = tmp; 
                        } 
                        ind += ngrid; 
printf("%d %f\n", k, alpha); 
                    } 
printf("%f %f %f\n", alpha, change, change/alpha); 
                alpha = change/alpha; ind = 0; 
                if (alpha < 1.0)  
                    for (k = 2; k < 5; k++) 
                        if (imsw[k]) { 
                            for (j = 0; j < ngrid; j++) 
                                image[j+index+ind] = old_image[j+index+ind]*(1.0-alpha)+image[j+index+ind]*alpha; 
                            ind += ngrid; 
                        } 
            } 
*/
            cp_image(Cq,Cm,Br,Bt,Bp,ngrid,image,imsw);
        }

        count++;
        Rsw = 0;
	calc_all = 0;
    }

    if (linear || spectroscopic) {
        cp_image(Cq,Cm,Br,Bt,Bp,ngrid,image,imsw);
        cp_data(Is,Qs,Us,Vs,spec_info,nobs,s_data);
    }

    untransform(Cq,Cm,ngrid,magnetic);
    save_bright(ngrid,Cq,Cm,Br,Bt,Bp,filename,1,com1,inc,ve,magnetic);
    if (batch == 1) printf("%s\n",filename);

    printf("Do you want to save the synthetic spectra (y/n) : ");
    scanf("%1s", answer);
    if ((answer[0] == 'Y') || (answer[0] == 'y')) {
        if (batch == 1) printf("yes\n");
        untransform_photometry(spec_info,nobs,Is,mag0,wphot); 
        save_spec(Is,Qs,Us,Vs,nobs,spec_info,filename,1,com3,1);
        if (batch == 1) printf("%s\n",filename);
    } else
        if (batch == 1) printf("no\n");

    end_spectral_lines();

    free_vector(C, 0, ngrid-1); 
    free_matrix(R,0,row-1,0,col-1);
    free_vector(image,0,col-1);
    free_vector(w,0,col-1);
    free_vector(m_data,0,row-1);
    free_vector(s_data,0,row-1);
    free_vector(sig2,0,row-1);
    free_svector((void *) spec_info);
    if (old_datsw[0]) free_vector(Im,0,npnsm1);
    if (old_datsw[1]) free_vector(Qm,0,npnsm1);
    if (old_datsw[2]) free_vector(Um,0,npnsm1);
    if (old_datsw[3]) free_vector(Vm,0,npnsm1);
    if (!linear)
        free_vector(old_image,0,col-1);
    if (datsw[0]) free_vector(Is,0,npnsm1);
    if (datsw[1]) free_vector(Qs,0,npnsm1);
    if (datsw[2]) free_vector(Us,0,npnsm1);
    if (datsw[3]) free_vector(Vs,0,npnsm1);
    free_vector(Cq,0,ngrid-1);
    if (magnetic) {
        free_vector(Cm,0,ngrid-1);
        free_vector(Br,0,ngrid-1);
        free_vector(Bt,0,ngrid-1);
        free_vector(Bp,0,ngrid-1);
    }
    free_ivector(line_used,0,nlines-1);
    if (spectroscopic) {
        free_vector(old_data,0,row-1);
	free_vector(cont,0,nobs-1);
	if (ndc) free_vector(dcont,0,ndc-1);
    }
}


void check_available_lines(spec_info,nobs,line_names,line_ids,nlines,line_used)
spec_info_t spec_info[];
int nobs,nlines,line_ids[],line_used[];
char *line_names[];
{
    int j,k,found,pre_broadened,linear;
    double lam0,dummy;

    for (k = 0; k < nlines; k++)
	line_used[k] = 0;

    for (j = 0; j < nobs; j++) {
	found = 0;
	for (k = 0; k < nlines; k++)
	    if (!strcmp(spec_info[j].line_name,line_names[k])) {
		found = 1;
	        spec_info[j].line_id = line_ids[k];
		line_used[k] = 1;
		break;
            }
        if (!found) {
	    spec_info[j].selected = 0;
	    printf("There is no line model available for observation %1d (%s).\n",
				                        j+1,spec_info[j].line_name);
        } else {
	    spec_info[j].selected = 1;
            line_info(spec_info[j].line_id,&lam0,&dummy,&pre_broadened,&linear);
	    spec_info[j].broadened = pre_broadened;
	    spec_info[j].linear = linear;
            if (fabs(lam0 - spec_info[j].line_centre) > 0.001*fabs(lam0)) {
                printf("WARNING : The centre wavelengths of the observed data at ");
                printf("observation %1d\n    and spectral model are not the same.\n",
							   j+1);
            }
        }
    }
}


void get_lines_to_fit(spec_info,nobs,line_names,line_used,nlines,batch)
spec_info_t spec_info[];
int nobs,nlines,line_used[],batch;
char *line_names[];
{
    int j,k;
    char answer[2];

    for (k = 0; k < nlines; k++)
	if (line_used[k]) {
	    printf("Do you want to use the %s observations (y/n) : ",line_names[k]);
	    scanf("%1s",answer);
            if ((answer[0] == 'Y') || (answer[0] == 'y')) {
		if (batch) printf("yes\n");
            } else {
                if (batch) printf("no\n");
		for (j = 0; j < nobs; j++)
	            if (!strcmp(spec_info[j].line_name,line_names[k]))
	                spec_info[j].selected = 0;
            }
	}
}


void get_stokes_params_to_fit(spec_info,nobs,datsw,batch)
spec_info_t spec_info[];
int nobs,datsw[4],batch;
{
    int j,k,selected,dummy;

    get_selected_stokes(spec_info,nobs,datsw);
    get_datsw(datsw,&dummy,batch);
    for (j = 0; j < nobs; j++)
	if (spec_info[j].selected) {
	    selected = 0;
	    for (k = 0; k < 4; k++)
		if (spec_info[j].datsw[k])
		    if (datsw[k]) {
		        selected = 1;
			spec_info[j].datsel[k] = 1;
                    } else
			spec_info[j].datsel[k] = 0;
            if (!selected)
		spec_info[j].selected = 0;
        }
}


static void set_pixel_weights(w,weight,ngrid,n_image)
float w[];
int weight,ngrid,n_image;
{
    int i,j;
    float *area;

    if (weight) {
	area = vector(0,ngrid-1);
	areas(ngrid,area);
	for (j = 0; j < n_image; j++)
	    for (i = 0; i < ngrid; i++) {
		w[i+j*ngrid] = area[i];
            }
	free_vector(area,0,ngrid-1);
    } else {
	for (i = 0; i < ngrid*n_image; i++)
	    w[i] = 1.0;
    }
}


int get_photometry_info(spec_info_t *spec_info, int nobs, float *mag0, float *wphot, 
                        int batch)
{
    int n, j; 

    for (j = n = 0; j < nobs; j++) 
        if (spec_info[j].photometric && spec_info[j].selected) n++; 

    if (n) {
        printf("Unspotted V magnitude : "); 
        scanf("%f", mag0); 
        if (batch) printf("%.3f\n", *mag0); 

        printf("Weight on photometry : "); 
        scanf("%f", wphot); 
        if (batch) printf("%.3f\n", *wphot); 
    } else 
        *wphot = 1.0;  
    return n; 
}
